"""
@author: Sandro Tanner
28.01.2023

CONTENT
1. Import                                                                                       65
2. Create OD - pairs                                                                            88
3. Web Scraping                                                                                 118
4. Parameters                                                                                   376
5. Join Datasets                                                                                396
6. Air Travel: City and airport combinations, joining scraped data, total travel time           451
7. Rail Travel: joining scraped data, total travel time                                         594
8. Car Travel Costs                                                                             609
9. Data Analysis                                                                                631
10. Mode Choice TMC Functions                                                                   768
11. Mode Choice Parameters and Value of Travel Time                                             905
12. Initial Modal Split                                                                         937
13. Modal Split under TMC                                                                       1.182
14. Analysis & Graphs                                                                           1.403
15. Maps                                                                                        1.693
"""

globals().clear()  # delete all variables

calendar_date = "22_12_2022"

import pandas as pd
from tabulate import tabulate
import numpy as np
import os
import time
from selenium import webdriver
from selenium.webdriver.chrome.options import Options
from selenium.webdriver.support.ui import WebDriverWait
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.common.by import By
from selenium.webdriver.common.keys import Keys  # added
from selenium.common.exceptions import NoSuchElementException
from selenium.common.exceptions import TimeoutException
import pyautogui
import matplotlib as mpl
import matplotlib.pyplot as plt
import statistics
from sklearn.linear_model import LinearRegression
import six
import scipy
from scipy import optimize
from scipy import stats
import math
import requests
import os
import pprint as pp
import sqlite3 as sq
import googlemaps
import statistics
import seaborn as sns
import geopy.distance
from mpl_toolkits import mplot3d
import geopandas
from mpl_toolkits.basemap import Basemap
import networkx as nx
import colorsys
from matplotlib.patches import Rectangle

######################################### 1. Import #############################################

cities = pd.read_csv("C:/Users/super/Documents/Masterarbeit/Model/cities.csv", sep=";")
coordinates = pd.read_csv("C:/Users/super/Documents/Masterarbeit/Model/coordinates.csv", sep=";")
od_pairs = pd.read_csv("C:/Users/super/Documents/Masterarbeit/Model/all od-pairs.csv", sep=";")
city_airports = pd.read_csv("C:/Users/super/Documents/Masterarbeit/Model/city_airports.csv", sep=";",encoding='unicode_escape')
od_pairs_airports = pd.read_csv("C:/Users/super/Documents/Masterarbeit/Model/all od-pairs-airports.csv", sep=";")
od_pairs_airports_cities = pd.read_csv("C:/Users/super/Documents/Masterarbeit/Model/all od-pairs-airports.csv", sep=";")
city_airports_unique = city_airports.iloc[:,2:].drop_duplicates()  # all possible airport connections, independent from which city passenger travels

pax_flow = pd.read_csv("C:/Users/super/Documents/Masterarbeit/Data/Jorik Grolle/edited/European_CityToCity_all_Passengers_leisure.csv",sep=";")
pax_flow_air = pd.read_csv("C:/Users/super/Documents/Masterarbeit/Data/Jorik Grolle/edited/European_CityToCity_air_Passengers_leisure.csv",sep=";")
fuel_price_per_country = pd.read_csv("C:/Users/super/Documents/Masterarbeit/Model/fuel prices.csv", sep=";")
distance_road = pd.read_csv("C:/Users/super/Documents/Masterarbeit/Data/Jorik Grolle/edited/distance_road.csv", sep=";")
duration_road = pd.read_csv("C:/Users/super/Documents/Masterarbeit/Data/Jorik Grolle/edited/duration_road.csv", sep=";")
city_to_airport_time = pd.read_csv("C:/Users/super/Documents/Masterarbeit/Data//Jorik Grolle/edited/#City_to_Airport_access_and_egress_time.csv",sep=";", encoding='unicode_escape')
city_to_airport_distance = pd.read_csv("C:/Users/super/Documents/Masterarbeit/Data//Jorik Grolle/edited/#City_to_Airport_access_and_egress_distance.csv",sep=";", encoding='unicode_escape')

data_credits = pd.read_csv("C:/Users/super/Documents/Masterarbeit/Credit Price/credit_price.csv", sep=";")
data_emissions = pd.read_csv("C:/Users/super/Documents/Masterarbeit/Model/CO2_emissions.csv",sep=";")  # emissions per passenger-km


######################################### 2. Create OD-pairs #########################################

amount_of_cities = len(cities)
amount_of_connections = len(city_airports)
amount_of_airports = len(city_airports_unique)

# Write origin and destination cities
for i in range(len(cities["city"])):  # i is origin station
    for j in range(len(cities["city"])):  # j is destination station
        od_pairs["origin"].iloc[i * amount_of_cities + j] = cities["city"].iloc[i]
        od_pairs["origin country"].iloc[i * amount_of_cities + j] = cities["country"].iloc[i]
        od_pairs["destination"].iloc[i * amount_of_cities + j] = cities["city"].iloc[j]
        od_pairs["destination country"].iloc[i * amount_of_cities + j] = cities["country"].iloc[j]

# Write origin and destination airports
for i in range(amount_of_airports):  # i is origirpoin art
    for j in range(amount_of_airports):  # j is destination airport
        od_pairs_airports["origin airport"].iloc[i * amount_of_airports + j] = city_airports_unique["airport kiwi"].iloc[i]
        od_pairs_airports["origin city"].iloc[i * amount_of_airports + j] = city_airports_unique["city kiwi"].iloc[i]
        od_pairs_airports["origin country"].iloc[i * amount_of_airports + j] = city_airports_unique["country kiwi"].iloc[i]
        od_pairs_airports["origin airport code"].iloc[i * amount_of_airports + j] = city_airports_unique["airport code"].iloc[i]
        od_pairs_airports["destination airport"].iloc[i * amount_of_airports + j] = city_airports_unique["airport kiwi"].iloc[j]
        od_pairs_airports["destination city"].iloc[i * amount_of_airports + j] = city_airports_unique["city kiwi"].iloc[j]
        od_pairs_airports["destination country"].iloc[i * amount_of_airports + j] = city_airports_unique["country kiwi"].iloc[j]
        od_pairs_airports["destination airport code"].iloc[i * amount_of_airports + j] = city_airports_unique["airport code"].iloc[j]

        od_pairs_airports["lat origin"].iloc[i * amount_of_airports + j] = city_airports_unique["latitude"].iloc[i]
        od_pairs_airports["long origin"].iloc[i * amount_of_airports + j] = city_airports_unique["longitude"].iloc[i]
        od_pairs_airports["lat destination"].iloc[i * amount_of_airports + j] = city_airports_unique["latitude"].iloc[j]
        od_pairs_airports["long destination"].iloc[i * amount_of_airports + j] = city_airports_unique["longitude"].iloc[j]

######################################### 3. Web Scraping #########################################

# initialise the data scraping interface
options = Options()
options.binary_location = r'C:/Program Files/Google/Chrome/Application/chrome.exe'  # changed from Francescos code
# options.binary_location = r'C:/Program Files/Microsoft/Edge/Application/msedge.exe'   #changed from Francescos code
options.add_argument("--window-size=1500,800")

########################## Rail: trainline.com ##########################

for i in range(len(cities["city"])):  # i is origin airport
    for j in range(len(cities["city"])):  # j is destination airport

        if i == j:  # continue if origin and destination are the same
            continue
        elif i < 1:
            continue
        else:
            # Trainline
            DRIVER_PATH = 'C:/Users/super/Documents/Masterarbeit/Web Scraping/chromedriver.exe'  # changed from Francescos code
            # DRIVER_PATH = 'C:/Users/super/Documents/Masterarbeit/Web Scraping/msedgedriver.exe'  # changed from Francescos code

            driver = webdriver.Chrome(options=options, executable_path=DRIVER_PATH)
            # driver = webdriver.Edge(options=options, executable_path=DRIVER_PATH)

            origin = cities["trainline code"].iloc[i]
            destination = cities["trainline code"].iloc[j]
            date = "2023-01-31T10%3A00%3A00"

            try:
                driver.get(
                    "https://www.thetrainline.com/book/results?origin=" + origin + "&destination=" + destination + "&outwardDate=" + date + "&outwardDateType=departAfter&journeySearchType=single&passengers%5B%5D=1995-10-26%7C252cf41f-d524-4154-bd96-a7b183b38bde&directSearch=false&lang=nl&selectedOutward=Qe%2FF4YBqtLo%3D%3ADBaSn5KPg9c%3D%2C%2FDdA7vb80RQ%3D%3AStandard")
                time.sleep(2)
                privacy = driver.find_element("xpath", '//*[@id="onetrust-accept-btn-handler"]')
                driver.execute_script("arguments[0].click();", privacy)
                time.sleep(25)

                # Collect price, duration and amount of transfers of 10 connections
                price_list = []
                time_list = []
                transfer_list = []
                for s in range(10):
                    k = s + 2
                    k = str(k)
                    try:
                        price = driver.find_element("xpath",
                                                    '//*[@id="app"]/div/div[1]/main/div/div[1]/div/div[2]/div/div[' + k + ']/div/div[3]/div[1]/div/div/div[1]')
                        price_text = price.text  # Extract text from this xpath
                        price_text = price_text[2:]  # Remove €
                        price_float = float(price_text.replace(',', '.'))  # Replace "," with "." and convert to string
                        price_list.append(price_float)

                        travel_time = driver.find_element("xpath",
                                                          '//*[@id="app"]/div/div[1]/main/div/div[1]/div/div[2]/div/div[' + k + ']/div/div[1]/div[2]/div/span[1]')
                        time_text = travel_time.text
                        if time_text[1] == "u":
                            time_hour = float(time_text[0])  # 1-9 hours (one digit for hour)
                            time_minute = float(time_text[3:-1])
                        else:
                            time_hour = float(time_text[0:2])  # 10 or more hours (two digits for hour)
                            time_minute = float(time_text[4:-1])
                        time_total = time_hour * 60 + time_minute
                        time_list.append(time_total)

                        transfer = driver.find_element("xpath",
                                                       '//*[@id="app"]/div/div[1]/main/div/div[1]/div/div[2]/div/div[' + k + ']/div/div[1]/div[2]/div/span[2]')
                        transfer_text = transfer.text[0]
                        transfer_int = int(transfer_text)
                        transfer_list.append(transfer_int)

                    except:
                        pass
            except:
                pass

            # Check if already some connections are retrieved or if more need to be loaded
            if len(price_list) < 6:
                try:
                    show_later = driver.find_element("xpath",
                                                     '//*[@id="app"]/div/div[1]/main/div/div[1]/div/div[2]/button[2]')  # also show later connections
                    driver.execute_script("arguments[0].click();", show_later)
                    time.sleep(20)

                    price_list = []
                    time_list = []
                    transfer_list = []
                    for s in range(10):
                        k = s + 2
                        k = str(k)
                        try:
                            price = driver.find_element("xpath",
                                                        '//*[@id="app"]/div/div[1]/main/div/div[1]/div/div[2]/div/div[' + k + ']/div/div[3]/div[1]/div/div/div[1]')
                            price_text = price.text  # Extract text from this xpath
                            price_text = price_text[2:]  # Remove €
                            price_float = float(price_text.replace(',', '.'))  # Replace "," with "." and convert to string
                            price_list.append(price_float)

                            travel_time = driver.find_element("xpath",
                                                              '//*[@id="app"]/div/div[1]/main/div/div[1]/div/div[2]/div/div[' + k + ']/div/div[1]/div[2]/div/span[1]')
                            time_text = travel_time.text
                            if time_text[1] == "u":
                                time_hour = float(time_text[0])  # 1-9 hours (one digit for hour)
                                time_minute = float(time_text[3:-1])
                            else:
                                time_hour = float(time_text[0:2])  # 10 or more hours (two digits for hour)
                                time_minute = float(time_text[4:-1])
                            time_total = time_hour * 60 + time_minute
                            time_list.append(time_total)

                            transfer = driver.find_element("xpath",
                                                           '//*[@id="app"]/div/div[1]/main/div/div[1]/div/div[2]/div/div[' + k + ']/div/div[1]/div[2]/div/span[2]')
                            transfer_text = transfer.text[0]
                            transfer_int = int(transfer_text)
                            transfer_list.append(transfer_int)

                        except:
                            pass
                except:
                    pass

            # Choose best connection (= highest utility)
            utility_list = []
            transfer_weight = -60  # per transfer
            time_weight = -1  # per min
            cost_weight = -5  # per €, assuming value of travel time of 12 € per hour (60 * -1 / -5)

            for q in range(len(transfer_list)):
                utility = transfer_list[q] * transfer_weight + time_list[q] * time_weight + cost_weight * price_list[q]
                utility_list.append(utility)

            try:
                connection = utility_list.index(
                    max(utility_list))  # index of best connection (=connection with highest utility)

                od_pairs["TC rail"].iloc[amount_of_cities * i + j] = price_list[connection]
                od_pairs["TT rail"].iloc[amount_of_cities * i + j] = time_list[connection]
                od_pairs["transfers rail"].iloc[amount_of_cities * i + j] = transfer_list[connection]
                print(od_pairs["TC rail"].iloc[amount_of_cities * i + j])
            except:
                pass

            # Save as csv from time to time
            if j == 0 or j == 40 or j == 80:
                od_pairs.to_csv("C:/Users/super/Documents/Masterarbeit/Model/Export/od-pairs_rail_14_12_2022.csv")

            driver.close()

########################## Air: kiwi.com ##########################
DRIVER_PATH = 'C:/Users/super/Documents/Masterarbeit/Web Scraping/chromedriver.exe'  # changed from Francescos code
driver = webdriver.Chrome(options=options, executable_path=DRIVER_PATH)
driver.get("https://www.kiwi.com/nl")
time.sleep(1)
privacy = driver.find_element("xpath",
                              '//*[@id="cookies_accept"]/div/div')  # Full x-path because short x-path changes everytime
driver.execute_script("arguments[0].click();", privacy)

for i in range(amount_of_airports):  # i is origin airport
    for j in range(amount_of_airports):  # j is destination airport

        if i == j:  # continue if origin and destination are the same
            continue
        elif i < 52:
            continue
        elif i == 52 and j < 99:
            continue

        else:
            price_float = 1000000
            time_total = 1000000

            origin_airport = city_airports_unique["airport kiwi"].iloc[i]
            try:
                origin_airport = origin_airport.lower()  # remove capital letters
            except:
                pass

            origin_city = city_airports_unique["city kiwi"].iloc[i]
            try:
                origin_city = origin_city.lower()  # remove capital letters
            except:
                pass

            origin_country = city_airports_unique["country kiwi"].iloc[i]
            try:
                origin_country = origin_country.lower()
            except:
                pass

            destination_airport = city_airports_unique["airport kiwi"].iloc[j]
            try:
                destination_airport = destination_airport.lower()
            except:
                pass

            destination_city = city_airports_unique["city kiwi"].iloc[j]
            try:
                destination_city = destination_city.lower()
            except:
                pass

            destination_country = city_airports_unique["country kiwi"].iloc[j]
            try:
                destination_country = destination_country.lower()
            except:
                pass

            date = "2022-12-20"

            try:
                driver.get(
                    "https://www.kiwi.com/us/search/results/" + origin_airport + "-" + origin_city + "-" + origin_country + "/" + destination_airport + "-" + destination_city + "-" + destination_country + "/" + date + "/no-return?stopNumber=1~true")  # only direct flights
                print(
                    "https://www.kiwi.com/us/search/results/" + origin_airport + "-" + origin_city + "-" + origin_country + "/" + destination_airport + "-" + destination_city + "-" + destination_country + "/" + date + "/no-return?stopNumber=1~true")
                time.sleep(20)
            except:
                pass

            # Collect price, duration and amount of transfers of 10 connections
            price_list = []
            time_list = []
            transfer_list = []
            for s in range(10):
                k = s + 1
                k = str(k)
                try:
                    # price = driver.find_element("xpath", '/html/body/div[2]/div[2]/div[3]/div/div/div/div/div/div[2]/div/div[2]/div/div/div/div/div['+k+']/div/div/div[2]/div[1]/div/strong/span')
                    price = driver.find_element("xpath",
                                                '/html/body/div[2]/div[2]/div[4]/div/div/div/div/div/div[2]/div/div[1]/div[2]/div/div[1]/div[2]/span')
                    price_text = price.text  # Extract text from this xpath
                    price_text = price_text[0:-2]  # Remove €
                    price_float = float(price_text.replace(',', '.'))  # Replace "," with "." and convert to string
                    # price_list.append(price_float)

                    travel_time = driver.find_element("xpath",
                                                      '/html/body/div[2]/div[2]/div[4]/div/div/div/div/div/div[2]/div/div[1]/div[2]/div/div[1]/div[2]/time')
                    time_text = travel_time.text
                    if time_text[1] == "h":
                        time_hour = float(time_text[0])  # 1-9 hours (one digit for hour)
                        time_minute = float(time_text[3:-1])
                    else:
                        time_hour = float(time_text[0:1])  # 10 or more hours (two digits for hour)
                        time_minute = float(time_text[4:-1])
                    time_total = time_hour * 60 + time_minute
                    # time_list.append(time_total)
                except:
                    pass

            print(price_float)
            print(time_total)

            od_pairs_airports["TC air"].iloc[amount_of_airports * i + j] = price_float
            od_pairs_airports["TT air"].iloc[amount_of_airports * i + j] = time_total

            # Save as csv from time to time
            if j == 0 or j == 40 or j == 80:
                od_pairs_airports.to_csv(
                    "C:/Users/super/Documents/Masterarbeit/Model/Export/od-pairs-airports_02_11_20_12_4.csv")

######################################### 4. Parameters #########################################

#Detour Factor Rail
detour_factor_rail = 1.3

# Car Cost
car_occupancy = 1.9  # (Verkehrsverhalten der Bevölkerung, 2015)
fuel_consumption = 7 / 100  # liter per km
avg_fuel_cost = 1.8
toll_cost = 0.06  # € per km
car_cost = toll_cost + avg_fuel_cost * fuel_consumption

# Income elasticity
e = 1.4     # deducted from (Christensen & Nielsen, 2017)

# calculate car emissions per person from emissions per vehicle
for i in range(0, data_emissions.shape[0]):  # shape[0]=number of rows
    data_emissions["car person"].iloc[i] = round(data_emissions["car vehicle"].iloc[i] / car_occupancy, 3)


######################################### 5. Join Datasets #########################################

# Passenger Volume
for i in range(len(cities)):
    for j in range(len(cities)):
        r = i + 5  # row in pax volume table
        c = j + 5  # column in pax volume table
        od_pairs["volume"].iloc[i * amount_of_cities + j] = round(float(pax_flow.iloc[r, c]))
        od_pairs["new volume"].iloc[i * amount_of_cities + j] = round(float(pax_flow.iloc[r, c]))  # for later (elastic demand)
        od_pairs["air volume"].iloc[i * amount_of_cities + j] = round(float(pax_flow_air.iloc[r, c]))

# Car Travel Distance
for i in range(amount_of_cities):
    for j in range(amount_of_cities):
        row = i * amount_of_cities + j
        od_pairs["distance car"].iloc[row] = int(distance_road.iloc[i + 4, j + 5])

# Car Travel Time
for i in range(amount_of_cities):
    for j in range(amount_of_cities):
        row = i * amount_of_cities + j
        od_pairs["TT car"].iloc[row] = int(float(duration_road.iloc[i + 4, j + 5]) * 60)

# Longitude and Latitude, Air Distance between cities
for i in range(amount_of_cities):
    for j in range(amount_of_cities):
        row = i * amount_of_cities + j
        od_pairs["lat origin"].iloc[row] = distance_road["Unnamed: 3"].iloc[i + 4]
        od_pairs["long origin"].iloc[row] = distance_road["COUNTRY"].iloc[i + 4]
        od_pairs["lat destination"].iloc[row] = distance_road.iloc[2, j + 5]
        od_pairs["long destination"].iloc[row] = distance_road.iloc[3, j + 5]

        coords_origin = (od_pairs["lat origin"].iloc[row], od_pairs["long origin"].iloc[row])
        coords_destination = (od_pairs["lat destination"].iloc[row], od_pairs["long destination"].iloc[row])
        od_pairs["distance air"].iloc[row] = round(geopy.distance.geodesic(coords_origin, coords_destination).km)

# Rail Travel Distance
od_pairs["distance rail"] = round(od_pairs["distance air"] * detour_factor_rail)

# Air access/egress times and distances
for i in range(len(city_airports)):
    for j in range(len(city_to_airport_time)):
        for k in range(len(city_to_airport_time.columns)):
            if city_airports["city"].iloc[i] == \
                    city_to_airport_time["Access / Egress times between main European Airports and Cities"].iloc[j]:
                if city_airports["airport code"].iloc[i] == city_to_airport_time.iloc[0, k]:
                    try:
                        city_airports["access time"].iloc[i] = round(float(city_to_airport_time.iloc[j, k]), 2) * 60
                    except:
                        pass
                    try:
                        city_airports["access distance"].iloc[i] = round(float(city_to_airport_distance.iloc[j, k]), 2)
                    except:
                        pass

######################################### 6. Air Travel: City and airport combinations, joining scraped data, total travel time #########################################

# Write all combinations of cities and airports
for i in range(amount_of_connections):  # i is origin airport
    for j in range(amount_of_connections):  # j is destination airport
        od_pairs_airports_cities["origin city"].iloc[i * amount_of_connections + j] = city_airports["city"].iloc[i]
        od_pairs_airports_cities["origin airport"].iloc[i * amount_of_connections + j] = \
            city_airports["airport kiwi"].iloc[i]
        od_pairs_airports_cities["origin airport code"].iloc[i * amount_of_connections + j] = \
            city_airports["airport code"].iloc[i]
        od_pairs_airports_cities["origin country"].iloc[i * amount_of_connections + j] = \
            city_airports["country kiwi"].iloc[i]
        od_pairs_airports_cities["destination city"].iloc[i * amount_of_connections + j] = city_airports["city"].iloc[j]
        od_pairs_airports_cities["destination airport"].iloc[i * amount_of_connections + j] = \
            city_airports["airport kiwi"].iloc[j]
        od_pairs_airports_cities["destination airport code"].iloc[i * amount_of_connections + j] = \
            city_airports["airport code"].iloc[j]
        od_pairs_airports_cities["destination country"].iloc[i * amount_of_connections + j] = \
            city_airports["country kiwi"].iloc[j]
        if i != j:
            od_pairs_airports_cities["access time air"].iloc[i * amount_of_connections + j] = \
                city_airports["access time"].iloc[i]
            od_pairs_airports_cities["egress time air"].iloc[i * amount_of_connections + j] = \
                city_airports["access time"].iloc[j]
            od_pairs_airports_cities["access distance air"].iloc[i * amount_of_connections + j] = \
                city_airports["access distance"].iloc[i]
            od_pairs_airports_cities["egress distance air"].iloc[i * amount_of_connections + j] = \
                city_airports["access distance"].iloc[j]

            od_pairs_airports_cities["lat origin"].iloc[i * amount_of_connections + j] = \
                city_airports["latitude"].iloc[i]
            od_pairs_airports_cities["lat destination"].iloc[i * amount_of_connections + j] = \
                city_airports["latitude"].iloc[j]
            od_pairs_airports_cities["long origin"].iloc[i * amount_of_connections + j] = \
                city_airports["longitude"].iloc[i]
            od_pairs_airports_cities["long destination"].iloc[i * amount_of_connections + j] = \
                city_airports["longitude"].iloc[j]


# Join Scraped Flight Data with all combinations of cities and airports
od_pairs_flight_tickets = pd.read_csv(
    "C:/Users/super/Documents/Masterarbeit/Model/Export/od-pairs-airports_02_11_20_12_final.csv", sep=";")

for i in range(len(od_pairs_flight_tickets)):
    for j in range(len(od_pairs_airports_cities)):
        if od_pairs_flight_tickets["origin airport"].iloc[i] == od_pairs_airports_cities["origin airport"].iloc[j]:
            if od_pairs_flight_tickets["destination airport"].iloc[i] == \
                    od_pairs_airports_cities["destination airport"].iloc[j]:
                od_pairs_airports_cities["TT air"].iloc[j] = od_pairs_flight_tickets["TT air"].iloc[i]
                od_pairs_airports_cities["TC air"].iloc[j] = od_pairs_flight_tickets["TC air"].iloc[i]

# remove prices with cents (not correct data)
for i in range(len(od_pairs_airports_cities)):
    try:
        if od_pairs_airports_cities["TC air"].iloc[i] / math.floor(od_pairs_airports_cities["TC air"].iloc[i]) != 1:
            od_pairs_airports_cities["TC air"].iloc[i] = 10000000
    except:
        pass

# Calculate total air travel time and costs
airport_waiting_time = 110
od_pairs_airports_cities["TT air total"] = od_pairs_airports_cities["TT air"] + od_pairs_airports_cities[
    "access time air"] + od_pairs_airports_cities["egress time air"] + airport_waiting_time
# new column for total travel cost air
TC_air_total = [0] * len(od_pairs_airports_cities)  # generalised cost for each connection (generate new column)
od_pairs_airports_cities["TC air total"] = TC_air_total
od_pairs_airports_cities["TC air total"] = od_pairs_airports_cities["TC air"] + car_cost * (od_pairs_airports_cities["access distance air"] + od_pairs_airports_cities["egress distance air"])

od_pairs_airports_cities.to_csv("C:/Users/super/Documents/Masterarbeit/Model/Final_Results_22_12_2022/od_pairs_airports_cities.csv")

# Select best airport for each connection
generalised_cost_air = [0] * amount_of_connections ** 2  # generalised cost for each connection (generate new column)
od_pairs_airports_cities["generalised_cost_air"] = generalised_cost_air
vtt = 11.82  # €

# Generalised Costs, assuming that access and egress to airport by car
od_pairs_airports_cities["generalised_cost_air"] = od_pairs_airports_cities["TT air total"] / 60 * vtt + \
                                                   od_pairs_airports_cities["access distance air"] * car_cost + \
                                                   od_pairs_airports_cities["egress distance air"] * car_cost + \
                                                   od_pairs_airports_cities["TC air"]

# Generate columns in od_pairs for air access and egress distance, for lat and long origin and destination, for origin and destination airport
access_distance_air = [0] * len(od_pairs)
od_pairs["access distance air"] = access_distance_air
egress_distance_air = [0] * len(od_pairs)
od_pairs["egress distance air"] = egress_distance_air

lat_origin = [0] * len(od_pairs)
od_pairs["lat origin airport"] = lat_origin
long_origin = [0] * len(od_pairs)
od_pairs["long origin airport"] = long_origin
lat_destination = [0] * len(od_pairs)
od_pairs["lat destination airport"] = lat_destination
long_destination = [0] * len(od_pairs)
od_pairs["long destination airport"] = long_destination

origin_airports = [0] * len(od_pairs)
od_pairs["origin airport"] = origin_airports
destination_airports = [0] * len(od_pairs)
od_pairs["destination airport"] = destination_airports

# Select airport with lowest generalised cost
for j in range(amount_of_cities):
    for k in range(amount_of_cities):
        if j != k:
            try:
                origin_city = cities["city"].iloc[j]
                destination_city = cities["city"].iloc[k]
                these_connections = od_pairs_airports_cities[od_pairs_airports_cities["origin city"] == origin_city]
                these_connections = these_connections[these_connections["destination city"] == destination_city]
                lowest_generalised_cost_air = these_connections["generalised_cost_air"].idxmin()
                # Write values in od_pairs table:
                od_pairs["TT air"].iloc[j * amount_of_cities + k] = od_pairs_airports_cities["TT air total"].iloc[
                    lowest_generalised_cost_air]
                od_pairs["TC air"].iloc[j * amount_of_cities + k] = od_pairs_airports_cities["TC air total"].iloc[
                    lowest_generalised_cost_air]
                od_pairs["access distance air"].iloc[j * amount_of_cities + k] = od_pairs_airports_cities["access distance air"].iloc[lowest_generalised_cost_air]
                od_pairs["egress distance air"].iloc[j * amount_of_cities + k] = od_pairs_airports_cities["egress distance air"].iloc[lowest_generalised_cost_air]

                od_pairs["lat origin airport"].iloc[j * amount_of_cities + k] = od_pairs_airports_cities["lat origin"].iloc[lowest_generalised_cost_air]
                od_pairs["long origin airport"].iloc[j * amount_of_cities + k] = od_pairs_airports_cities["long origin"].iloc[lowest_generalised_cost_air]
                od_pairs["lat destination airport"].iloc[j * amount_of_cities + k] = od_pairs_airports_cities["lat destination"].iloc[lowest_generalised_cost_air]
                od_pairs["long destination airport"].iloc[j * amount_of_cities + k] = od_pairs_airports_cities["long destination"].iloc[lowest_generalised_cost_air]

                od_pairs["origin airport"].iloc[j * amount_of_cities + k] = od_pairs_airports_cities["origin airport"].iloc[lowest_generalised_cost_air]
                od_pairs["destination airport"].iloc[j * amount_of_cities + k] = od_pairs_airports_cities["destination airport"].iloc[lowest_generalised_cost_air]

            except:
                pass
#od_pairs_backup = od_pairs
#od_pairs = od_pairs.dropna(subset="origin country")
#od_pairs=od_pairs_backup

# Update air distance (distance between chosen airports)
for i in range(len(od_pairs)):
    coords_origin = (od_pairs["lat origin airport"].iloc[i], od_pairs["long origin airport"].iloc[i])
    coords_destination = (od_pairs["lat destination airport"].iloc[i], od_pairs["long destination airport"].iloc[i])
    try:
        od_pairs["distance air"].iloc[i] = round(geopy.distance.geodesic(coords_origin, coords_destination).km)
    except:
        pass


######################################### 7. Rail Travel: joining scraped data, total travel time #########################################

# od_pairs = od_pairs_complete
od_pairs_rail = pd.read_csv("C:/Users/super/Documents/Masterarbeit/Model/Export/od-pairs_new_08_11_added_rail.csv",
                            sep=";")

od_pairs["TT rail"] = od_pairs_rail["TT rail"]
od_pairs["TC rail"] = od_pairs_rail["TC rail"]
od_pairs["transfers rail"] = od_pairs_rail["transfers rail"]

station_access_time = 20
station_waiting_time = 15
od_pairs["TT rail"] = od_pairs["TT rail"] + 2 * station_access_time + station_waiting_time


######################################### 8. Car Travel Costs ##################################################

# fuel costs dependent on origin and destination country
fuel_price_list = [0] * len(od_pairs)
od_pairs["fuel price"] = fuel_price_list
for i in range(len(od_pairs)):
    origin_country = od_pairs["origin country"].iloc[i]
    destination_country = od_pairs["destination country"].iloc[i]
    fuel_price_origin = \
        fuel_price_per_country.loc[fuel_price_per_country["country"] == origin_country]["Nov 22"].values[0]
    fuel_price_destination = \
        fuel_price_per_country.loc[fuel_price_per_country["country"] == destination_country]["Nov 22"].values[0]
    this_fuel_price = (fuel_price_origin + fuel_price_destination) / 1000 / 2
    od_pairs["fuel price"].iloc[i] = this_fuel_price

# car travel cost is fuel cost plus toll cost
od_pairs["TC car"] = (fuel_consumption * od_pairs["fuel price"] * od_pairs["distance car"] + toll_cost * od_pairs[
    "distance car"]) / car_occupancy


od_pairs.to_csv("C:/Users/super/Documents/Masterarbeit/Model/Final_Results_22_12_2022/od_pairs_complete_before_modal_split.csv")

######################################### 9. Data Analysis #########################################

# remove OD-pairs without data
od_pairs = od_pairs[od_pairs["TC air"] > 0]
od_pairs = od_pairs[od_pairs["TC air"] < 10000000]
od_pairs = od_pairs[od_pairs["TC rail"] > 0]
od_pairs = od_pairs[od_pairs["TT air"] > 0]

# remove OD-pairs without passenger volume data
od_pairs = od_pairs[od_pairs["volume"] > 0]

# remove Pristina (no data)
od_pairs = od_pairs[od_pairs["destination"] != "Pristina"]

# round car costs
od_pairs["TC car"] = round(od_pairs["TC car"])

od_pairs.to_csv("C:/Users/super/Documents/Masterarbeit/Model/Final_Results_22_12_2022/od_pairs_filtered_before_modal_split.csv")
od_pairs = pd.read_csv("C:/Users/super/Documents/Masterarbeit/Model/Final_Results_22_12_2022/od_pairs_filtered_before_modal_split.csv")


# Calculate statistical indicators
TC_air_avg = round(statistics.mean(od_pairs["TC air"]))
TC_air_median = round(statistics.median(od_pairs["TC air"]))
TC_air_std = round(statistics.stdev(od_pairs["TC air"]))
TC_air_max = round(od_pairs["TC air"].max())
TC_air_min = round(od_pairs["TC air"].min())
TC_rail_avg = round(statistics.mean(od_pairs["TC rail"]))
TC_rail_median = round(statistics.median(od_pairs["TC rail"]))
TC_rail_std = round(statistics.stdev(od_pairs["TC rail"]))
TC_rail_max = round(od_pairs["TC rail"].max())
TC_rail_min = round(od_pairs["TC rail"].min())
TC_car_avg = round(statistics.mean(od_pairs["TC car"]))
TC_car_median = round(statistics.median(od_pairs["TC car"]))
TC_car_std = round(statistics.stdev(od_pairs["TC car"]))
TC_car_max = round(od_pairs["TC car"].max())
TC_car_min = round(od_pairs["TC car"].min())

TC_avg = [TC_air_avg, TC_rail_avg, TC_car_avg]
TC_median = [TC_air_median, TC_rail_median, TC_car_median]
TC_std = [TC_air_std, TC_rail_std, TC_car_std]
TC_max = [TC_air_max, TC_rail_max, TC_car_max]
TC_min = [TC_air_min, TC_rail_min, TC_car_min]

TT_air_avg = statistics.mean(od_pairs["TT air"])
TT_air_median = round(statistics.median(od_pairs["TT air"]))
TT_air_std = round(statistics.stdev(od_pairs["TT air"]))
TT_air_max = round(od_pairs["TT air"].max())
TT_air_min = round(od_pairs["TT air"].min())
TT_rail_avg = round(statistics.mean(od_pairs["TT rail"]))
TT_rail_median = round(statistics.median(od_pairs["TT rail"]))
TT_rail_std = round(statistics.stdev(od_pairs["TT rail"]))
TT_rail_max = round(od_pairs["TT rail"].max())
TT_rail_min = round(od_pairs["TT rail"].min())
TT_car_avg = round(statistics.mean(od_pairs["TT car"]))
TT_car_median = round(statistics.median(od_pairs["TT car"]))
TT_car_std = round(statistics.stdev(od_pairs["TT car"]))
TT_car_max = round(od_pairs["TT car"].max())
TT_car_min = round(od_pairs["TT car"].min())

TT_avg = [TT_air_avg, TT_rail_avg, TT_car_avg]
TT_median = [TT_air_median, TT_rail_median, TT_car_median]
TT_std = [TT_air_std, TT_rail_std, TT_car_std]
TT_max = [TT_air_max, TT_rail_max, TT_car_max]
TT_min = [TT_air_min, TT_rail_min, TT_car_min]

# Density Plot Travel Cost
sns.kdeplot(od_pairs["TC air"], label="Air", color="mediumblue", legend=True)
sns.kdeplot(od_pairs["TC rail"], label="Rail", color="crimson", legend=True)
sns.kdeplot(od_pairs["TC car"], label="Car", color="darkorange", legend=True)
plt.xlabel('Travel Cost [€]')
plt.ylabel('Density')
plt.xlim(0, 500)
plt.legend()

# Density Plot Travel Time
sns.kdeplot(od_pairs["TT air"], label="Air", color="mediumblue", legend=True, clip=(0, 1700))
sns.kdeplot(od_pairs["TT rail"], label="Rail", color="crimson", legend=True, clip=(0, 1700))
sns.kdeplot(od_pairs["TT car"], label="Car", color="darkorange", legend=True, clip=(0, 1700))
plt.xlabel('Travel Time [min]')
plt.ylabel('Density')
plt.legend()

# Density Plot Travel Distances
sns.kdeplot(od_pairs["distance air"], label="Air", color="mediumblue", legend=True)
sns.kdeplot(od_pairs["distance rail"], label="Rail", color="crimson", legend=True)
sns.kdeplot(od_pairs["distance car"], label="Car", color="darkorange", legend=True)
plt.xlabel('Travel Distance [km]')
plt.ylabel('Density')
plt.legend()

# Plot Travel Costs by distance
sns.scatterplot(x=od_pairs_elastic["distance air"], y=od_pairs_elastic["TC air"], label="Air", legend=True, color="mediumblue", s=5)
sns.scatterplot(x=od_pairs_elastic["distance air"], y=od_pairs_elastic["TC rail"], label="Rail", legend=True, color="crimson", s=5)
sns.scatterplot(x=od_pairs_elastic["distance air"], y=od_pairs_elastic["TC car"], label="Car", legend=True, color="darkorange", s=5)
plt.xlabel('Direct Distance [km]')
plt.ylabel('Travel Cost [€]')
plt.grid
plt.legend()
plt.savefig("C:/Users/super/Documents/Masterarbeit/Model/Final_Results_22_12_2022/Figures/travel_costs_by_distance.png", dpi=1200)

# Plot Travel Time by distance
sns.scatterplot(x=od_pairs_elastic["distance air"], y=od_pairs_elastic["TT air"], label="Air", legend=True, color="mediumblue", s=5)
sns.scatterplot(x=od_pairs_elastic["distance air"], y=od_pairs_elastic["TT rail"], label="Rail", legend=True, color="crimson", s=5)
sns.scatterplot(x=od_pairs_elastic["distance air"], y=od_pairs_elastic["TT car"], label="Car", legend=True, color="darkorange", s=5)
plt.xlabel('Direct Distance [km]')
plt.ylabel('Travel Time [min]')
plt.grid
plt.legend()
plt.savefig("C:/Users/super/Documents/Masterarbeit/Model/Final_Results_22_12_2022/Figures/travel_time_by_distance.png", dpi=1200)

# Violin plot travel costs
fig = plt.figure(figsize=(10, 4))
data_to_plot = [od_pairs["TC air"], od_pairs["TC rail"], od_pairs["TC car"]]
ax = sns.violinplot(data_to_plot, linewidth=0.7, width=0.8, palette=["mediumblue", "crimson", "darkorange"])
ax.set_xlabel('Air                                                      Rail                                                      Car')
ax.set_ylabel('Travel Cost [€]')
plt.savefig("C:/Users/super/Documents/Masterarbeit/Model/Final_Results_22_12_2022/Figures/violin_plot_travel_costs.png", dpi=1200)

# Violin plot travel time
fig = plt.figure(figsize=(10, 4))
data_to_plot = [od_pairs["TT air"], od_pairs["TT rail"], od_pairs["TT car"]]
ax = sns.violinplot(data_to_plot, linewidth=0.7, width=0.8, palette=["mediumblue", "crimson", "darkorange"])
ax.set_xlabel('Air                                                       Rail                                                       Car')
ax.set_ylabel('Travel Time [min]')
plt.savefig("C:/Users/super/Documents/Masterarbeit/Model/Final_Results_22_12_2022/Figures/violin_plot_travel_time.png", dpi=1200)
#ax.set_ylim(0, 1)

# Violin plot travel distance
fig = plt.figure(figsize=(10, 4))
data_to_plot = [od_pairs["distance air"] + od_pairs["access distance air"] + od_pairs["egress distance air"], od_pairs["distance rail"], od_pairs["distance car"]]
ax = sns.violinplot(data_to_plot, linewidth=0.7, width=0.8, palette=["mediumblue", "crimson", "darkorange"])
ax.set_xlabel('Air                                                       Rail                                                       Car')
ax.set_ylabel('Travel Distance [km]')
plt.savefig("C:/Users/super/Documents/Masterarbeit/Model/Final_Results_22_12_2022/Figures/violin_plot_travel_distance.png", dpi=1200)


######################################### 10. Mode Choice TMC Functions #########################################

def credit_expenditure(data_credits, e, emissions,
                       reduction_target):  # Calculate credit expenditure and credit need per country
    population_2019 = data_credits["population 2019"].sum()
    population_2023 = data_credits["population 2023"].sum()
    avg_credit_expenditure = emissions / population_2019

    avg_weighed_net_income = data_credits["net income"].sum() / len(data_credits["net income"])
    # Average weighed net income
    s = 0
    for i in range(0, data_credits.shape[0]):
        s = s + data_credits["net income"].iloc[i] * data_credits["population 2019"].iloc[i]
    avg_weighed_net_income = s / population_2023
    # Credit expenditure
    for i in range(0, data_credits.shape[0]):
        data_credits["credit expenditure"].iloc[i] \
            = e * (data_credits["net income"].iloc[
                       i] - avg_weighed_net_income) * avg_credit_expenditure / avg_weighed_net_income + avg_credit_expenditure
    # Credit allocation
    for i in range(0, data_credits.shape[0]):
        data_credits["credit allocation"].iloc[i] = initial_emission_all_OD_pairs / data_credits[
            "population 2019"].sum() * (1 - reduction_target)

    # Credit need
    for i in range(0, data_credits.shape[0]):
        data_credits["credit need"].iloc[i] = data_credits["credit expenditure"].iloc[i] - \
                                              data_credits["credit allocation"].iloc[i]

    return data_credits


def credit_valuation(data_credits):  # Calculate amount of credits bought and sold by people in each country
    for i in range(0, data_credits.shape[0]):
        data_credits["valuation"].iloc[i] = data_credits["WTP"].iloc[i] / data_credits["credit expenditure"].iloc[i]
    # Amount of credits bought by people in a country
    for i in range(0, data_credits.shape[0]):
        data_credits["credit buy"].iloc[i] = data_credits["credit need"].iloc[i] * data_credits["population 2019"].iloc[
            i]
    # Amount of credits sold by people in a country
    for i in range(0, data_credits.shape[0]):
        data_credits["credit sell"].iloc[i] = -data_credits["credit need"].iloc[i] * \
                                              data_credits["population 2019"].iloc[i]
    return data_credits


def buy_sell_quantity(data_credits):
    buy = data = data_credits[data_credits['credit buy'] > 0]
    buy_quantity = buy["credit buy"].sum()
    sell = data_credits[data_credits['credit sell'] > 0]
    sell_quantity = sell["credit sell"].sum()
    return buy_quantity, sell_quantity


def car_google_maps(lat_origin, long_origin, lat_destination,
                    long_destination):  # Gets travel time and distance data from Google Maps
    API_key = "AIzaSyBuHrSwv_8PWS-kKxEji1LWWaIAYpfSxfU"
    gmaps = googlemaps.Client(key=API_key)
    origin = (lat_origin, long_origin)
    destination = (lat_destination, long_destination)
    result = gmaps.distance_matrix(origin, destination, mode='driving')
    result_distance = result["rows"][0]["elements"][0]["distance"]["value"] / 1000
    result_time = result["rows"][0]["elements"][0]["duration"]["value"] / 60
    return result_distance, result_time


def utility(B_time, B_cost, TT, TC):
    U = B_time * TT + B_cost * TC
    return U


def logit(U1, U2, U3):
    P = math.exp(U1) / (math.exp(U1) + math.exp(U2) + math.exp(U3))
    return P


def pax_share_origin(value_of_travel_time, origin_country,
                     destination_country):  # Share of travellers from origin country and destination country (proportional to countries' populations)
    population_origin = value_of_travel_time.loc[value_of_travel_time["country"] == origin_country][
        "population"].values  # .values to remove first row (which counts rows)
    population_destination = value_of_travel_time.loc[value_of_travel_time["country"] == destination_country][
        "population"].values
    share_origin_country = population_origin / (population_origin + population_destination)[0]
    return share_origin_country


def VTT_country(value_of_travel_time, origin_country, destination_country, share_origin):
    VTT_leisure_origin = \
        value_of_travel_time.loc[value_of_travel_time["country"] == origin_country]["VTT leisure"].values[0]
    VTT_leisure_destination = \
        value_of_travel_time.loc[value_of_travel_time["country"] == destination_country]["VTT leisure"].values[0]
    VTT_leisure = share_origin * VTT_leisure_origin + (1 - share_origin) * VTT_leisure_destination
    return VTT_leisure


def credits_for_trip(data_emissions, distance_air, distance_rail, distance_car):  # Calculates credits needed per trip
    if distance_air < 400:
        credit_air = data_emissions["air regional"].iloc[0] * distance_air / 1000  # ton per pax km
    elif distance_air > 1000:
        credit_air = data_emissions["air european"].iloc[0] * distance_air / 1000  # ton per pax km
    else:  # linear decrease of emissions if distance between 400 and 1000 (in data, at 700km emissions are lower)
        a = (data_emissions["air european"].iloc[0] - data_emissions["air regional"].iloc[0]) / (
                1000 - 400)  # slope of linear function
        b = (data_emissions["air european"].iloc[0] * 400 - data_emissions["air regional"].iloc[
            0] * 1000) / -600  # intersect with y-axis of linear function
        credit_air = (a * distance_air + b) * distance_air / 1000
    credit_rail = data_emissions["rail IC"].iloc[0] * distance_rail / 1000  # ton per pax km
    credit_car = data_emissions["car person"].iloc[0] * distance_car / 1000  # ton per pax km
    return credit_air, credit_rail, credit_car


def credits_for_trip_final(credit_amount,
                           credit_need):  # Assuming that people need to buy credits only if they dont have enough allocated (need to buy the difference of what they need for this trip and have)
    if credit_need > 0:  # people have to buy all credits for this air trip
        credit_amount_final = credit_amount  # credit cost need fully be paid
    if credit_need < 0 and abs(
            credit_need) > credit_amount:  # people dont need to buy any credits (they have more credits left than they need for this trip)
        credit_amount_final = 0
    if credit_need < 0 and abs(
            credit_need) < credit_amount:  # people need to buy part of the credits they need for their trip
        credit_amount_final = credit_amount - abs(credit_need)
    return credit_amount_final


def credit_cost_for_trip(credit_cost,
                         credit_price):  # Calculates credit cost per trip !!!! erweitern dass Country übergeben wird!!!!!!!
    credit_cost = credit_cost * credit_price
    return credit_cost


def avg_credit_cost_for_trip(credit_cost_o, credit_cost_d, volume,
                             share_origin):  # Average credit costs (depends on share of nationality of passengers and their credit costs)
    credit_cost_avg = (volume * share_origin * credit_cost_o + volume * (1 - share_origin) * credit_cost_d) / volume
    return credit_cost_avg



#################################### 11. Mode Choice Parameters and Value of Travel Time #################################

# ALTERNATIVE-SPECIFIC VARIABLES ("The paired combinatorial logit model: properties, estimation and application")
B_cost_initial = -0.0461
B_time_initial = -0.0099  # [min]
VTT_initial = B_time_initial / B_cost_initial * 60

# Value of Travel Time per country
value_of_travel_time = pd.read_csv("C:/Users/super/Documents/Masterarbeit/model/value_of_travel_time.csv", sep=";")
value_of_travel_time["VTT"] = value_of_travel_time["GDP per capita"] / value_of_travel_time["working hours"]

avg_VTT = 0
for i in range(len(value_of_travel_time)):
    avg_VTT = avg_VTT + value_of_travel_time["VTT"].iloc[i] * value_of_travel_time["population"].iloc[i]
avg_VTT = avg_VTT / value_of_travel_time["population"].sum()

VTT_europe = 17.86  # Values of travel time in Europe: Review and meta-analysis
VTT_europe_leisure = 11.82  # Values of travel time in Europe: Review and meta-analysis
VTT_leisure_ratio = VTT_europe_leisure / VTT_europe
value_of_travel_time["VTT leisure"] = round(value_of_travel_time["VTT"] * VTT_leisure_ratio, 2)

avg_VTT_leisure = 0
for i in range(len(value_of_travel_time)):
    avg_VTT_leisure = avg_VTT_leisure + value_of_travel_time["VTT leisure"].iloc[i] * \
                      value_of_travel_time["population"].iloc[i]
avg_VTT_leisure = avg_VTT_leisure / value_of_travel_time["population"].sum()

value_of_travel_time.to_csv(
    "C:/Users/super/Documents/Masterarbeit/Model/intermediate_outputs/value_of_travel_time_" + calendar_date + ".csv")



############################################ 12. Initial Modal Split ############################################

# Create empty list
VTT_list = []

initial_emission_all_OD_pairs = 0  # initiate variable for initial emissions of considered OD-pairs

# Calculate initial modal split for all OD-pairs,
for j in range(0, od_pairs.shape[0]):
    B_cost = B_cost_initial
    B_time = B_time_initial  # [min]

    origin_country = od_pairs["origin country"].iloc[j]
    destination_country = od_pairs["destination country"].iloc[j]
    origin_city = od_pairs["origin"].iloc[j]
    destination_city = od_pairs["destination"].iloc[j]
    volume = od_pairs["volume"].iloc[j]
    distance_air = od_pairs["distance air"].iloc[j]
    distance_rail = od_pairs["distance rail"].iloc[j]
    distance_car = od_pairs["distance car"].iloc[j]
    access_distance_air = od_pairs["access distance air"].iloc[j]
    egress_distance_air = od_pairs["egress distance air"].iloc[j]
    TT_air = od_pairs["TT air"].iloc[j]
    TT_rail = od_pairs["TT rail"].iloc[j]
    TT_car = od_pairs["TT car"].iloc[j]
    TC_air = od_pairs["TC air"].iloc[j]
    TC_rail = od_pairs["TC rail"].iloc[j]
    TC_car = od_pairs["TC car"].iloc[j]

    # Share of travellers from origin country and destination country (proportional to countries' populations)
    share_origin = pax_share_origin(value_of_travel_time, origin_country, destination_country)[0]

    # Value of Travel time, dependent on share of passengers from origin and destination country
    VTT_leisure = VTT_country(value_of_travel_time, origin_country, destination_country, share_origin)
    VTT_list.append(VTT_leisure)

    B_time = B_time * VTT_leisure / VTT_initial

    # Initial Utility
    U_air = utility(B_time, B_cost, TT_air, TC_air)
    U_rail = utility(B_time, B_cost, TT_rail, TC_rail)
    U_car = utility(B_time, B_cost, TT_car, TC_car)

    # Initial Modal Share
    P_air = logit(U_air, U_rail, U_car)
    P_rail = logit(U_rail, U_air, U_car)
    P_car = logit(U_car, U_air, U_rail)

    # Initial CO2 emissions for this route. Emissions per person times volume by air [ton CO2]
    emissions_air = credits_for_trip(data_emissions, distance_air, distance_rail, distance_car)[0] * volume * P_air + \
                    credits_for_trip(data_emissions, distance_air, distance_rail, access_distance_air)[2] * volume * P_air + \
                    credits_for_trip(data_emissions, distance_air, distance_rail, egress_distance_air)[2] * volume * P_air  # Flight emissions plus access and egress emissions with car
    emissions_rail = credits_for_trip(data_emissions, distance_air, distance_rail, distance_car)[1] * volume * P_rail
    emissions_car = credits_for_trip(data_emissions, distance_air, distance_rail, distance_car)[2] * volume * P_car
    emissions_total = emissions_air + emissions_rail + emissions_car

    initial_emission_all_OD_pairs = initial_emission_all_OD_pairs + emissions_total

    # Save values to dataframe
    od_pairs["modal share air"].iloc[j] = P_air
    od_pairs["modal share rail"].iloc[j] = P_rail
    od_pairs["modal share car"].iloc[j] = P_car
    od_pairs["emissions air"].iloc[j] = emissions_air
    od_pairs["emissions rail"].iloc[j] = emissions_rail
    od_pairs["emissions car"].iloc[j] = emissions_car

od_pairs["VTT leisure"] = VTT_list

od_pairs.to_csv(
    "C:/Users/super/Documents/Masterarbeit/Model/Final_Results_22_12_2022/od-pairs_initial_modal_share.csv")
od_pairs = pd.read_csv("C:/Users/super/Documents/Masterarbeit/Model/Final_Results_22_12_2022/od-pairs_initial_modal_share.csv")


# Calculate statistical indicators
modal_share_air_avg = round(statistics.mean(od_pairs["modal share air"]), 2)
modal_share_air_median = round(statistics.median(od_pairs["modal share air"]), 2)
modal_share_air_std = round(statistics.stdev(od_pairs["modal share air"]), 2)
modal_share_air_max = round(od_pairs["modal share air"].max(), 2)
modal_share_air_min = round(od_pairs["modal share air"].min(), 2)
modal_share_rail_avg = round(statistics.mean(od_pairs["modal share rail"]), 2)
modal_share_rail_median = round(statistics.median(od_pairs["modal share rail"]), 2)
modal_share_rail_std = round(statistics.stdev(od_pairs["modal share rail"]), 2)
modal_share_rail_max = round(od_pairs["modal share rail"].max(), 2)
modal_share_rail_min = round(od_pairs["modal share rail"].min(), 2)
modal_share_car_avg = round(statistics.mean(od_pairs["modal share car"]), 2)
modal_share_car_median = round(statistics.median(od_pairs["modal share car"]), 2)
modal_share_car_std = round(statistics.stdev(od_pairs["modal share car"]), 2)
modal_share_car_max = round(od_pairs["modal share car"].max(), 2)
modal_share_car_min = round(od_pairs["modal share car"].min(), 2)

modal_share_avg = [modal_share_air_avg, modal_share_rail_avg, modal_share_car_avg]
modal_share_median = [modal_share_air_median, modal_share_rail_median, modal_share_car_median]
modal_share_std = [modal_share_air_std, modal_share_rail_std, modal_share_car_std]
modal_share_max = [modal_share_air_max, modal_share_rail_max, modal_share_car_max]
modal_share_min = [modal_share_air_min, modal_share_rail_min, modal_share_car_min]

emissions_air_avg = round(statistics.mean(od_pairs["emissions air"]))
emissions_air_median = round(statistics.median(od_pairs["emissions air"]))
emissions_air_std = round(statistics.stdev(od_pairs["emissions air"]))
emissions_air_max = round(od_pairs["emissions air"].max())
emissions_air_min = round(od_pairs["emissions air"].min())
emissions_rail_avg = round(statistics.mean(od_pairs["emissions rail"]))
emissions_rail_median = round(statistics.median(od_pairs["emissions rail"]))
emissions_rail_std = round(statistics.stdev(od_pairs["emissions rail"]))
emissions_rail_max = round(od_pairs["emissions rail"].max())
emissions_rail_min = round(od_pairs["emissions rail"].min())
emissions_car_avg = round(statistics.mean(od_pairs["emissions car"]))
emissions_car_median = round(statistics.median(od_pairs["emissions car"]))
emissions_car_std = round(statistics.stdev(od_pairs["emissions car"]))
emissions_car_max = round(od_pairs["emissions car"].max())
emissions_car_min = round(od_pairs["emissions car"].min())

emissions_avg = [emissions_air_avg, emissions_rail_avg, emissions_car_avg]
emissions_median = [emissions_air_median, emissions_rail_median, emissions_car_median]
emissions_std = [emissions_air_std, emissions_rail_std, emissions_car_std]
emissions_max = [emissions_air_max, emissions_rail_max, emissions_car_max]
emissions_min = [emissions_air_min, emissions_rail_min, emissions_car_min]

# Density Plot Modal Share
sns.kdeplot(od_pairs["modal share air"], label="air", color="mediumblue", legend=True)
sns.kdeplot(od_pairs["modal share rail"], label="rail", color="crimson", legend=True)
sns.kdeplot(od_pairs["modal share car"], label="car", color="darkorange", legend=True)
plt.xlabel('Modal Share')
plt.ylabel('Density')
plt.legend()

# Plot Modal Share by distance
plt.figure(figsize=(10, 6))
sns.regplot(x=od_pairs["distance air"], y=od_pairs["modal share air"], label="air", color="mediumblue",
            scatter_kws={'s': 2}, order=3, ci=99.9)
sns.regplot(x=od_pairs["distance air"], y=od_pairs["modal share rail"], label="rail", color="crimson",
            scatter_kws={'s': 2}, order=3, ci=99.9)
sns.regplot(x=od_pairs["distance air"], y=od_pairs["modal share car"], label="car", color="darkorange",
            scatter_kws={'s': 2}, order=3, ci=99.9)
plt.xlabel('Air Distance [km]')
plt.ylabel('Modal Share')
plt.ylim(0, 1)
plt.xlim(150, 1500)
plt.legend()        #loc="center left", bbox_to_anchor=(0.5, 0.5))
plt.savefig("C:/Users/super/Documents/Masterarbeit/Model/Final_Results_22_12_2022/Figures/modal_share_by_distance.png", dpi=1200)


# Pax Flow
# Set up base map
fig = plt.figure(figsize=(55, 45))
m = Basemap(projection='lcc', resolution='l', area_thresh=1000,
            lon_0=10, lat_0=49, lat_1=37, lat_2=61,
            width=3.4E6, height=3E6)
# draw map elements
m.drawcoastlines(color='#999999')
m.fillcontinents(color='#333333', lake_color='#999999')
m.drawmapboundary(fill_color='#999999')
m.drawcountries(color='#999999', linewidth=1.0)
# import long lat as m attribute
for i in range(len(cities)):
    lon = cities["longitude"].iloc[i]
    lat = cities["latitude"].iloc[i]
    xpt, ypt = m(lon, lat)
    dot_size = cities["pax flow total"].iloc[i] / 4000000
    #dot_size = math.log(cities["pax flow total"].iloc[i] / 10000)
    lonpt, latpt = m(xpt, ypt, inverse=True)
    m.plot(xpt, ypt, 'bo', markersize=dot_size, color="orange")

# Amount of connections from/to city (to and from added up)
# Set up base map
fig = plt.figure(figsize=(55, 45))
m = Basemap(projection='lcc', resolution='l', area_thresh=1000,
            lon_0=10, lat_0=49, lat_1=37, lat_2=61,
            width=3.4E6, height=3E6)
# draw map elements
m.drawcoastlines(color='#999999')
m.fillcontinents(color='#333333', lake_color='#999999')
m.drawmapboundary(fill_color='#999999')
m.drawcountries(color='#999999', linewidth=1.0)
# import long lat as m attribute
for i in range(len(cities)):
    lon = cities["longitude"].iloc[i]
    lat = cities["latitude"].iloc[i]
    city_name = cities["city"].iloc[i]
    xpt, ypt = m(lon, lat)
    od_pairs_connected_origin = od_pairs[od_pairs["origin"] == city_name]
    od_pairs_connected_destination = od_pairs[od_pairs["destination"] == city_name]
    dot_size = (len(od_pairs_connected_origin) + len(od_pairs_connected_destination)) / 10
    #dot_size = math.log(cities["pax flow total"].iloc[i] / 10000)
    lonpt, latpt = m(xpt, ypt, inverse=True)
    m.plot(xpt, ypt, 'bo', markersize=dot_size, color="orange")

# Values of Travel Time for trips starting in Switzerland
od_pairs_switzerland = od_pairs[od_pairs["origin country"] == "Switzerland"]
for i in range(len(od_pairs_switzerland) - 1):
    if od_pairs_switzerland["destination country"].iloc[i] == od_pairs_switzerland["destination country"].iloc[i + 1]:
        od_pairs_switzerland["VTT leisure"].iloc[i + 1] = 0
od_pairs_switzerland = od_pairs_switzerland[od_pairs_switzerland["VTT leisure"] != 0]
od_pairs_switzerland = od_pairs_switzerland.sort_values(by=["VTT leisure"], ascending=False)
sns.barplot(x=od_pairs_switzerland["destination country"], y=od_pairs_switzerland["VTT leisure"],
            label="Value of Travel Time", color="mediumblue")
plt.xlabel('Destination')
plt.ylabel('Value of Travel Time [min]')
plt.xticks(rotation=90)
plt.legend()

# Values of Travel Time for trips starting in the Netherlands
od_pairs_netherlands = od_pairs[od_pairs["origin country"] == "The Netherlands"]
for i in range(len(od_pairs_netherlands) - 1):
    if od_pairs_netherlands["destination country"].iloc[i] == od_pairs_netherlands["destination country"].iloc[i + 1]:
        od_pairs_netherlands["VTT leisure"].iloc[i + 1] = 0
od_pairs_netherlands = od_pairs_netherlands[od_pairs_netherlands["VTT leisure"] != 0]
od_pairs_netherlands = od_pairs_netherlands.sort_values(by=["VTT leisure"], ascending=False)
sns.barplot(x=od_pairs_netherlands["destination country"], y=od_pairs_netherlands["VTT leisure"],
            label="Value of Travel Time", color="green")
plt.xlabel('Destination')
plt.ylabel('Value of Travel Time [min]')
plt.xticks(rotation=90)
plt.legend()
plt.savefig("C:/Users/super/Documents/Masterarbeit/Model/Final_Results_22_12_2022/Figures/VTT_Netherlands.png", dpi=1200)


# Values of Travel Time for trips starting in Germany
od_pairs_germany = od_pairs[od_pairs["origin country"] == "Germany"]
for i in range(len(od_pairs_germany) - 1):
    if od_pairs_germany["destination country"].iloc[i] == od_pairs_germany["destination country"].iloc[i + 1]:
        od_pairs_germany["VTT leisure"].iloc[i + 1] = 0
od_pairs_germany = od_pairs_germany[od_pairs_germany["VTT leisure"] != 0]
od_pairs_germany = od_pairs_germany.sort_values(by=["VTT leisure"], ascending=False)
sns.barplot(x=od_pairs_germany["destination country"], y=od_pairs_germany["VTT leisure"], label="Value of Travel Time",
            color="mediumblue")
plt.xlabel('Destination')
plt.ylabel('Value of Travel Time [min]')
plt.xticks(rotation=90)
plt.legend()

# Fuel price for trips starting in Switzerland
od_pairs_switzerland = od_pairs[od_pairs["origin country"] == "Switzerland"]
for i in range(len(od_pairs_switzerland) - 1):
    if od_pairs_switzerland["destination country"].iloc[i] == od_pairs_switzerland["destination country"].iloc[i + 1]:
        od_pairs_switzerland["fuel price"].iloc[i + 1] = 0
od_pairs_switzerland = od_pairs_switzerland[od_pairs_switzerland["fuel price"] != 0]
od_pairs_switzerland = od_pairs_switzerland.sort_values(by=["fuel price"], ascending=False)
sns.barplot(x=od_pairs_switzerland["destination country"], y=od_pairs_switzerland["fuel price"], label="Fuel Price",
            color="mediumblue")
plt.xlabel('Destination')
plt.ylabel('Fuel Price [€/liter]')
plt.xticks(rotation=90)
plt.legend()

######################################## 13. Modal Split under TMC ############################################

# Create empty lists
credit_price_list = []  # here values of every iteration will be saved
gap_list = []  # gap between bought and sold credits will be saved
emission_change_percentual_list = []
cancel_rate_air_list = []
cancel_rate_rail_list = []
cancel_rate_car_list = []
credit_expenditure_list = pd.DataFrame(data_credits["credit expenditure"])
TC_air_new_list =[]

#new columns for new travel costs
TC_air_new_list = [0] * len(od_pairs)
od_pairs["TC air new"] = TC_air_new_list
TC_rail_new_list = [0] * len(od_pairs)
od_pairs["TC rail new"] = TC_rail_new_list
TC_car_new_list = [0] * len(od_pairs)
od_pairs["TC car new"] = TC_car_new_list

#Choose reduction target
reduction_target = 0.8

budget = (1 - reduction_target) * initial_emission_all_OD_pairs
total_emissions = initial_emission_all_OD_pairs

# Choose Number of iterations
number_of_iterations = 15

# Choose if demand is elastic?
elastic = False

credit_price = 0

for i in range(number_of_iterations):  # Run code several times (so far until there is an equilibrium credit price)
    new_emission_all_OD_pairs = 0  # inititate variable for new emissions of considered OD-pairs

    # Calculation of Credit Price

    data_credits = credit_expenditure(data_credits, e, total_emissions, reduction_target)

    data_credits = credit_valuation(data_credits)

     # Check if there is more buyers or more sellers. If more buyers (gap positive), credit price must increase, if more sellers (gap negative) it must decrease
    buy_quantity = buy_sell_quantity(data_credits)[0]
    sell_quantity = buy_sell_quantity(data_credits)[1]
    gap = buy_quantity - sell_quantity  # amount of missing sellers (positive if too many buyers, negative if too many sellers)
    gap_share = gap / (buy_quantity + sell_quantity)  # ratio between gap and sum of credits to be sold and bought
    if elastic == False:
        credit_price = credit_price + 30000 * gap_share
    else:
        credit_price = credit_price + 100 * gap_share

    for j in range(0, od_pairs.shape[0]):
        # ALTERNATIVE-SPECIFIC VARIABLES ("The paired combinatorial logit model: properties, estimation and application")
        B_cost = B_cost_initial
        B_time = B_time_initial  # [min]

        volume = od_pairs["volume"].iloc[j]

        origin_country = od_pairs["origin country"].iloc[j]
        destination_country = od_pairs["destination country"].iloc[j]

        distance_air = od_pairs["distance air"].iloc[j]
        distance_rail = od_pairs["distance rail"].iloc[j]
        distance_car = od_pairs["distance car"].iloc[j]
        access_distance_air = od_pairs["access distance air"].iloc[j]
        egress_distance_air = od_pairs["egress distance air"].iloc[j]
        TT_air = od_pairs["TT air"].iloc[j]
        TT_rail = od_pairs["TT rail"].iloc[j]
        TT_car = od_pairs["TT car"].iloc[j]
        TC_air = od_pairs["TC air"].iloc[j]
        TC_rail = od_pairs["TC rail"].iloc[j]
        TC_car = od_pairs["TC car"].iloc[j]

        # Share of travellers from origin country and destination country (proportional to countries' populations)
        share_origin = pax_share_origin(value_of_travel_time, origin_country, destination_country)[0]

        # Value of Travel time, dependent on share of passengers from origin and destination country
        VTT_leisure = VTT_country(value_of_travel_time, origin_country, destination_country, share_origin)
        B_time = B_time * VTT_leisure / VTT_initial

        credit_air = credits_for_trip(data_emissions, distance_air, distance_rail, distance_car)[0] + \
                     credits_for_trip(data_emissions, distance_air, distance_rail, access_distance_air)[2] + \
                     credits_for_trip(data_emissions, distance_air, distance_rail, egress_distance_air)[2]  # Flight emissions plus access and egress emissions with car
        credit_rail = credits_for_trip(data_emissions, distance_air, distance_rail, distance_car)[1]
        credit_car = credits_for_trip(data_emissions, distance_air, distance_rail, distance_car)[2]

        credit_cost_air = credit_cost_for_trip(credit_air, credit_price)
        credit_cost_rail = credit_cost_for_trip(credit_rail, credit_price)
        credit_cost_car = credit_cost_for_trip(credit_car, credit_price)

        # Purchasing credits

        # Travel costs including credit costs
        TC_air_new = TC_air + credit_cost_air
        TC_rail_new = TC_rail + credit_cost_rail
        TC_car_new = TC_car + credit_cost_car

        od_pairs["TC air new"].iloc[j] = TC_air_new
        od_pairs["TC rail new"].iloc[j] = TC_rail_new
        od_pairs["TC car new"].iloc[j] = TC_car_new

        # Utility including credit costs
        U_air_new = utility(B_time, B_cost, TT_air, TC_air_new)
        U_rail_new = utility(B_time, B_cost, TT_rail, TC_rail_new)
        U_car_new = utility(B_time, B_cost, TT_car, TC_car_new)

        # Modal Share including credit costs
        P_air_new = logit(U_air_new, U_rail_new, U_car_new)
        P_rail_new = logit(U_rail_new, U_air_new, U_car_new)
        P_car_new = logit(U_car_new, U_air_new, U_rail_new)

        od_pairs["new modal share air"].iloc[j] = P_air_new
        od_pairs["new modal share rail"].iloc[j] = P_rail_new
        od_pairs["new modal share car"].iloc[j] = P_car_new

        od_pairs["modal shift air"].iloc[j] = od_pairs["new modal share air"].iloc[j] - \
                                              od_pairs["modal share air"].iloc[j]
        od_pairs["modal shift rail"].iloc[j] = od_pairs["new modal share rail"].iloc[j] - \
                                               od_pairs["modal share rail"].iloc[j]
        od_pairs["modal shift car"].iloc[j] = od_pairs["new modal share car"].iloc[j] - \
                                              od_pairs["modal share car"].iloc[j]

        # Trip cancellation
        if elastic == True:
            price_elasticity = (-0.278 - 0.544) / 2  # (de Bok et al., 2010)
            # Q2 = e * (P2-P1) / P1 * Q1 + Q1

            Q1_air = volume * P_air_new
            Q1_rail = volume * P_rail_new
            Q1_car = volume * P_car_new
            Q2_air = price_elasticity * (TC_air_new - TC_air) / TC_air * Q1_air + Q1_air
            if Q2_air < 0:  # for the case demand becomes negative
                Q2_air = 0
            Q2_rail = price_elasticity * (TC_rail_new - TC_rail) / TC_rail * Q1_rail + Q1_rail
            if Q2_rail < 0:  # for the case demand becomes negative
                Q2_rail = 0
            Q2_car = price_elasticity * (TC_car_new - TC_car) / TC_car * Q1_car + Q1_car
            if Q2_car < 0:  # for the case demand becomes negative
                Q2_car = 0
            cancel_rate_air = (Q1_air - Q2_air) / Q1_air
            cancel_rate_rail = (Q1_rail - Q2_rail) / Q1_rail
            cancel_rate_car = (Q1_car - Q2_car) / Q1_car
            od_pairs["cancel rate air"].iloc[j] = cancel_rate_air
            od_pairs["cancel rate rail"].iloc[j] = cancel_rate_rail
            od_pairs["cancel rate car"].iloc[j] = cancel_rate_car

            volume_elastic = Q2_air + Q2_rail + Q2_car
            od_pairs["new volume"].iloc[j] = volume_elastic

        # New CO2 emissions for this route
        if elastic == False:  # no demand elasticity
            emissions_air_new = credit_air * volume * P_air_new  # emissions per person times volume by air [ton CO2]
            emissions_rail_new = credit_rail * volume * P_rail_new
            emissions_car_new = credit_car * volume * P_car_new

        elif elastic == True:  # with demand elasticity
            emissions_air_new = credit_air * volume_elastic * P_air_new  # emissions per person times volume by air [ton CO2]
            emissions_rail_new = credit_rail * volume_elastic * P_rail_new
            emissions_car_new = credit_car * volume_elastic * P_car_new

        od_pairs["new emissions air"].iloc[j] = emissions_air_new
        od_pairs["new emissions rail"].iloc[j] = emissions_rail_new
        od_pairs["new emissions car"].iloc[j] = emissions_car_new
        od_pairs["emission change air"].iloc[j] = od_pairs["new emissions air"].iloc[j] - \
                                                  od_pairs["emissions air"].iloc[j]
        od_pairs["emission change rail"].iloc[j] = od_pairs["new emissions rail"].iloc[j] - \
                                                   od_pairs["emissions rail"].iloc[j]
        od_pairs["emission change car"].iloc[j] = od_pairs["new emissions car"].iloc[j] - \
                                                  od_pairs["emissions car"].iloc[j]

        emissions_total_new = emissions_air_new + emissions_rail_new + emissions_car_new

        new_emission_all_OD_pairs = new_emission_all_OD_pairs + emissions_total_new

        # Save values to dataframe
        od_pairs["emission change total"].iloc[j] = od_pairs["emission change air"].iloc[j] + \
                                                    od_pairs["emission change rail"].iloc[j] + \
                                                    od_pairs["emission change car"].iloc[
                                                        j]  # emission change of this OD-pair
        od_pairs["emission change total %"].iloc[j] = od_pairs["emission change total"].iloc[j] / (
                od_pairs["emissions air"].iloc[j] + od_pairs["emissions rail"].iloc[j] +
                od_pairs["emissions car"].iloc[j])

    emission_change_percentual = (new_emission_all_OD_pairs - initial_emission_all_OD_pairs) / initial_emission_all_OD_pairs * 100

    total_emissions = new_emission_all_OD_pairs

    # Save values to iteration list
    credit_price_list.append(credit_price)
    emission_change_percentual_list.append(emission_change_percentual)
    gap_list.append(gap)
    new_credit_expenditure = data_credits["credit expenditure"]
    credit_expenditure_list[i] = new_credit_expenditure  # save values of credit expenditure of all iterations

#new columns for relative change of modal share
rel_change_modal_share_air = [0] * len(od_pairs)
od_pairs["rel change modal share air"] = rel_change_modal_share_air
rel_change_modal_share_rail = [0] * len(od_pairs)
od_pairs["rel change modal share rail"] = rel_change_modal_share_rail
rel_change_modal_share_car = [0] * len(od_pairs)
od_pairs["rel change modal share car"] = rel_change_modal_share_car

od_pairs["rel change modal share air"] = (od_pairs["new modal share air"] - od_pairs["modal share air"]) / od_pairs["modal share air"]
od_pairs["rel change modal share rail"] = (od_pairs["new modal share rail"] - od_pairs["modal share rail"]) / od_pairs["modal share rail"]
od_pairs["rel change modal share car"] = (od_pairs["new modal share car"] - od_pairs["modal share car"]) / od_pairs["modal share car"]

#Save Results
od_pairs_inelastic = od_pairs
od_pairs_inelastic.to_csv("C:/Users/super/Documents/Masterarbeit/Model/Final_Results_22_12_2022/od_pairs_final_inelastic.csv")
od_pairs_inelastic = pd.read_csv("C:/Users/super/Documents/Masterarbeit/Model/Final_Results_22_12_2022/od_pairs_final_inelastic.csv")

od_pairs_elastic = od_pairs
od_pairs_elastic.to_csv("C:/Users/super/Documents/Masterarbeit/Model/Final_Results_22_12_2022/od_pairs_final_elastic.csv")
od_pairs_elastic = pd.read_csv("C:/Users/super/Documents/Masterarbeit/Model/Final_Results_22_12_2022/od_pairs_final_elastic.csv")

od_pairs_70percent = od_pairs
od_pairs_70percent.to_csv("C:/Users/super/Documents/Masterarbeit/Model/Final_Results_22_12_2022/different reduction targets inelastic/od_pairs_70percent.csv")


############################# 14. Analysis & Graphs #############################

# Analysis of evolution during iterations
iterations = pd.DataFrame(credit_price_list, columns=["credit price"])
indexes = list(range(0,number_of_iterations,1))
iterations["index"] = indexes
iterations["emission change percentual"] = emission_change_percentual_list
iterations["gap buy and sell"] = gap_list
print(iterations)

iteration_list = list(range(1, number_of_iterations + 1))
iterations["iteration"] = iteration_list
iterations.plot.scatter(x="iteration", y="emission change percentual", c="red")
plt.show()

# Plot emission change [%] evolution
x2 = iterations["index"]
plt.scatter(x=x2, y=iterations["emission change percentual"], c="red")
plt.plot(iterations["index"], iterations["emission change percentual"], c="red")
plt.xlabel('Iteration')
plt.ylabel('Emission change [%]')
matplotlib.pyplot.suptitle('Emission change [%] compared to 2019', fontsize=12)
plt.show()
plt.savefig("C:/Users/super/Documents/Masterarbeit/Model/Final_Results_22_12_2022/Figures/Emission_change_evolution_inelastic.png", dpi=1200)

# Plot Credit price evolution
plt.figure(0)
x1 = iterations["index"]
plt.scatter(x=x1, y=iterations["credit price"], c="green")
plt.plot(iterations["index"], iterations["credit price"], c="green")
plt.ylim(0, 4000)
plt.xlabel('Iteration')
plt.ylabel('Credit Price [€/ton CO2]')
matplotlib.pyplot.suptitle('Credit price', fontsize=12)
plt.show()
plt.savefig("C:/Users/super/Documents/Masterarbeit/Model/Final_Results_22_12_2022/Figures/Credit_price_evolution_inelastic.png", dpi=1200)

# Violin plot travel cost
fig = plt.figure(figsize=(10, 4))
data_to_plot = [od_pairs_elastic["TC air"], od_pairs_elastic["TC air new"], od_pairs_elastic["TC rail"],
                od_pairs_elastic["TC rail new"], od_pairs_elastic["TC car"], od_pairs_elastic["TC car new"]]
ax = sns.violinplot(data_to_plot, linewidth=0.7, width=1,
                    palette=["mediumblue", "mediumblue", "crimson", "crimson", "darkorange", "darkorange"])
ax.set_xlabel(
    'air without TMC     air with TMC     rail without TMC     rail with TMC     car without TMC     car with TMC')
ax.set_ylabel('travel costs [€]')

# Plot Modal shift by distance inelastic
plt.figure(figsize=(10, 6))
sns.scatterplot(x=od_pairs_inelastic["distance air"], y=od_pairs_inelastic["modal shift air"] * 100, label="air", legend=True, color="mediumblue",
                s=5)
sns.scatterplot(x=od_pairs_inelastic["distance air"], y=od_pairs_inelastic["modal shift rail"] * 100, label="rail", legend=True, color="crimson",
                s=5)
sns.scatterplot(x=od_pairs_inelastic["distance air"], y=od_pairs_inelastic["modal shift car"] * 100, label="car", legend=True, color="darkorange",
                s=5)
plt.grid
plt.xlabel('Air Distance [km]')
plt.ylabel('Modal Shift [%]')
plt.ylim(-100, 100)
plt.legend(framealpha=1)
plt.savefig("C:/Users/super/Documents/Masterarbeit/Model/Final_Results_22_12_2022/Figures/Modal_shift_by_distance_inelastic.png", dpi=1200)

# Plot Modal shift by distance elastic
plt.figure(figsize=(10, 6))
sns.scatterplot(x=od_pairs_elastic["distance air"], y=od_pairs_elastic["modal shift air"] * 100, label="air", legend=True, color="mediumblue",
                s=5)
sns.scatterplot(x=od_pairs_elastic["distance air"], y=od_pairs_elastic["modal shift rail"] * 100, label="rail", legend=True, color="crimson",
                s=5)
sns.scatterplot(x=od_pairs_elastic["distance air"], y=od_pairs_elastic["modal shift car"] * 100, label="car", legend=True, color="darkorange",
                s=5)
sns.set_style("ticks",{'axes.grid' : True})
plt.grid
plt.xlabel('Air Distance [km]')
plt.ylabel('Modal Shift [%]')
plt.ylim(-100, 100)
plt.legend(framealpha=1)
plt.savefig("C:/Users/super/Documents/Masterarbeit/Model/Final_Results_22_12_2022/Figures/Modal_shift_by_distance_elastic.png", dpi=1200)

# Plot Modal Share by distance inelastic
plt.figure(figsize=(10, 6))
sns.regplot(x=od_pairs_inelastic["distance air"], y=od_pairs_inelastic["new modal share air"], label="air", color="mediumblue",
            scatter_kws={'s': 2}, order=3, ci=99.9)
sns.regplot(x=od_pairs_inelastic["distance air"], y=od_pairs_inelastic["new modal share rail"], label="rail", color="crimson",
            scatter_kws={'s': 2}, order=3, ci=99.9)
sns.regplot(x=od_pairs_inelastic["distance air"], y=od_pairs_inelastic["new modal share car"], label="car", color="darkorange",
            scatter_kws={'s': 2}, order=3, ci=99.9)
plt.xlabel('Direct Distance [km]')
plt.ylabel('Modal Share')
plt.ylim(0, 1)
plt.xlim(150, 1500)
plt.legend()    #loc="center left", bbox_to_anchor=(1, 0.5))
plt.savefig("C:/Users/super/Documents/Masterarbeit/Model/Final_Results_22_12_2022/Figures/Modal_share_by_distance_inelastic.png", dpi=1200)


# Plot Modal Share by distance elastic
plt.figure(figsize=(10, 6))
sns.regplot(x=od_pairs_elastic["distance air"], y=od_pairs_elastic["modal share air"] * 100, label="air", color="mediumblue",
            scatter_kws={'s': 2}, order=3, ci=99.9)
sns.regplot(x=od_pairs_elastic["distance air"], y=od_pairs_elastic["modal share rail"] * 100, label="rail", color="crimson",
            scatter_kws={'s': 2}, order=3, ci=99.9)
sns.regplot(x=od_pairs_elastic["distance air"], y=od_pairs_elastic["modal share car"] * 100, label="car", color="darkorange",
            scatter_kws={'s': 2}, order=3, ci=99.9)
sns.set_style("ticks",{'axes.grid' : True})
plt.xlabel('Air Distance [km]')
plt.ylabel('Modal Share [%]')
plt.ylim(0, 100)
plt.xlim(150, 1500)
plt.legend(framealpha=1)    #loc="center left", bbox_to_anchor=(1, 0.5))
plt.savefig("C:/Users/super/Documents/Masterarbeit/Model/Final_Results_22_12_2022/Figures/Modal_share_by_distance_100.png", dpi=1200)

# Plot Modal Share by distance, per mode
plt.figure(figsize=(10, 6))
sns.regplot(x=od_pairs_elastic["distance air"], y=od_pairs_elastic["modal share car"], label="Initial situation", color="Sienna",   #MidnightBlue, Sienna
            scatter_kws={'s': 2}, order=3, ci=99.9)
sns.regplot(x=od_pairs_elastic["distance air"], y=od_pairs_inelastic["new modal share car"], label="TMC with inelastic demand", color="OrangeRed",  #MediumBlue, OrangeRed
            scatter_kws={'s': 2}, order=3, ci=99.9)
sns.regplot(x=od_pairs_elastic["distance air"], y=od_pairs_elastic["new modal share car"], label="TMC with elastic demand", color="Orange", #DodgerBlue, Orange
            scatter_kws={'s': 2}, order=3, ci=99.9)
sns.set_style("ticks",{'axes.grid' : True})
plt.xlabel('Air Distance [km]')
plt.ylabel('Modal Share')
plt.ylim(0, 1)
plt.xlim(150, 1500)
plt.legend(loc= "upper left", framealpha=1)    #loc="center left", bbox_to_anchor=(1, 0.5))
plt.savefig("C:/Users/super/Documents/Masterarbeit/Model/Final_Results_22_12_2022/Figures/Modal_share_by_distance_car.png", dpi=1200)



# Modal Shift for trips starting in Zurich, sorted by initital modal share air (highest air share left, lowest air share right)
zurich = od_pairs[od_pairs["origin"] == "Zurich"]
zurich = zurich.sort_values(by=["modal share air"], ascending=False)
sns.barplot(x=zurich["destination"], y=od_pairs["modal shift rail"], label="air", color="mediumblue")
plt.xlabel('destination')
plt.ylabel('modal shift rail')
plt.legend(loc="center left", bbox_to_anchor=(1, 0.5))
plt.xticks(rotation=90)

# Plot Density Travel costs initial and with TMC
sns.kdeplot(od_pairs["TC air"], label="air", color="mediumblue", legend=True)
sns.kdeplot(od_pairs["TC air new"], label="air", color="mediumblue", legend=True)
sns.kdeplot(od_pairs["TC rail"], label="rail", color="crimson", legend=True)
sns.kdeplot(od_pairs["TC rail new"], label="rail", color="crimson", legend=True)
sns.kdeplot(od_pairs["TC car"], label="car", color="darkorange", legend=True)
sns.kdeplot(od_pairs["TC car new"], label="car", color="darkorange", legend=True)
plt.xlabel('Travel Cost [€]')
plt.ylabel('Density')
plt.legend()

# Plot Travel costs initial and with TMC by distance
sns.scatterplot(x=od_pairs["distance air"], y=od_pairs["TC air new"], label="air", legend=True, color="mediumblue", s=5)
sns.scatterplot(x=od_pairs["distance air"], y=od_pairs["TC rail new"], label="rail", legend=True, color="crimson", s=5)
sns.scatterplot(x=od_pairs["distance air"], y=od_pairs["TC car new"], label="car", legend=True, color="darkorange", s=5)
plt.xlabel('Travel Distance [km]')
plt.ylabel('Travel Cost [€]')
plt.legend()

# Plot Travel costs initial and with TMC by distance
sns.scatterplot(x=od_pairs["distance air"], y=od_pairs["TC air new"], label="air", legend=True, color="mediumblue", s=5)
sns.scatterplot(x=od_pairs["distance air"], y=od_pairs["TC rail new"], label="rail", legend=True, color="crimson", s=5)
sns.scatterplot(x=od_pairs["distance air"], y=od_pairs["TC car new"], label="car", legend=True, color="darkorange", s=5)
plt.xlabel('Travel Distance [km]')
plt.ylabel('Travel Cost [€]')
plt.legend()

# Plot trip cancel rate by distance
sns.scatterplot(x=od_pairs_elastic["distance air"], y=od_pairs_elastic["cancel rate air"], label="air", legend=True, color="mediumblue",
                s=5)
sns.scatterplot(x=od_pairs_elastic["distance air"], y=od_pairs_elastic["cancel rate rail"], label="rail", legend=True, color="crimson",
                s=5)
sns.scatterplot(x=od_pairs_elastic["distance air"], y=od_pairs_elastic["cancel rate car"], label="car", legend=True, color="darkorange",
                s=5)
plt.xlabel('Travel Distance [km]')
plt.ylabel('Trip Cancel Rate')
plt.legend()

# Plot trip cancel rate by initial travel cost for air
sns.scatterplot(x=od_pairs_elastic["TC air"], y=od_pairs_elastic["cancel rate air"] * 100, label="air", legend=True, color="mediumblue", s=5)
sns.scatterplot(x=od_pairs_elastic["TC rail"], y=od_pairs_elastic["cancel rate rail"] * 100, label="rail", legend=True, color="crimson", s=5)
sns.scatterplot(x=od_pairs_elastic["TC car"], y=od_pairs_elastic["cancel rate car"] * 100, label="car", legend=True, color="darkorange", s=5)
plt.xlabel('Initial Travel Cost [€]')
plt.ylabel('Trip Cancel Rate [%]')
plt.xlim(0, 300)
plt.ylim(0,100)
plt.legend()
plt.savefig("C:/Users/super/Documents/Masterarbeit/Model/Final_Results_22_12_2022/Figures/trip_cancel_rate_by_distance.png", dpi=1200)

# Plot trip cancel rate by initial emissions for air and rail (car left out because is constant, because emissions as well as costs and therewith cancel rate proportional to distance)
sns.scatterplot(x=1000*od_pairs_elastic["emissions air"]/(od_pairs_elastic["volume"]*od_pairs_elastic["modal share air"]), y=od_pairs_elastic["cancel rate air"] * 100, label="air", legend=True, color="mediumblue", s=5)
sns.scatterplot(x=1000*od_pairs_elastic["emissions rail"]/(od_pairs_elastic["volume"]*od_pairs_elastic["modal share rail"]), y=od_pairs_elastic["cancel rate rail"] * 100, label="rail", legend=True, color="crimson", s=5)
sns.scatterplot(x=1000*od_pairs_elastic["emissions car"]/(od_pairs_elastic["volume"]*od_pairs_elastic["modal share car"]), y=od_pairs_elastic["cancel rate car"] * 100, label="car", legend=True, color="darkorange", s=5)
plt.xlabel('Emissions per Trip [kg CO2]')
plt.ylabel('Trip Cancel Rate [%]')
plt.legend()
plt.savefig("C:/Users/super/Documents/Masterarbeit/Model/Final_Results_22_12_2022/Figures/trip_cancel_rate_by_emissions_per_trip.png", dpi=1200)

# Violin plot modal share
fig = plt.figure(figsize=(10, 4))
data_to_plot = [od_pairs_elastic["modal share air"], od_pairs_elastic["new modal share air"], od_pairs_elastic["modal share rail"],
                od_pairs_elastic["new modal share rail"], od_pairs_elastic["modal share car"], od_pairs_elastic["new modal share car"]]
ax = sns.violinplot(data_to_plot, linewidth=0.7, width=1.1,
                    palette=["blue", "mediumblue", "deeppink", "crimson", "orange", "darkorange"])
ax.set_xlabel(
    'air without TMC     air with TMC     rail without TMC     rail with TMC     car without TMC     car with TMC')
ax.set_ylabel('modal share')
ax.set_ylim(0, 1)

# Violin plot credit costs inelastic
fig = plt.figure(figsize=(10, 4))
data_to_plot = [od_pairs_inelastic["TC air new"] - od_pairs_inelastic["TC air"], od_pairs_inelastic["TC rail new"] - od_pairs_inelastic["TC rail"], od_pairs_inelastic["TC car new"] - od_pairs_inelastic["TC car"]]
ax = sns.violinplot(data_to_plot, linewidth=0.7, width=0.8,
                    palette=["mediumblue", "crimson", "darkorange"])
ax.set_xlabel('Air                                                       Rail                                                       Car')
ax.set_ylabel('Credit Costs [€]')
plt.savefig("C:/Users/super/Documents/Masterarbeit/Model/Final_Results_22_12_2022/Figures/violin_plot_credit_cost_inelastic.png", dpi=1200)

# Violin plot credit costs elastic
fig = plt.figure(figsize=(10, 4))
data_to_plot = [od_pairs_elastic["TC air new"] - od_pairs_elastic["TC air"], od_pairs_elastic["TC rail new"] - od_pairs_elastic["TC rail"], od_pairs_elastic["TC car new"] - od_pairs_elastic["TC car"]]
ax = sns.violinplot(data_to_plot, linewidth=0.7, width=0.8,
                    palette=["mediumblue", "crimson", "darkorange"])
ax.set_xlabel('Air                                                       Rail                                                       Car')
ax.set_ylabel('Credit Costs [€]')
plt.savefig("C:/Users/super/Documents/Masterarbeit/Model/Final_Results_22_12_2022/Figures/violin_plot_credit_cost_elastic.png", dpi=1200)


# Violin plot trip cancel rate air
fig = plt.figure(figsize=(5, 4))
data_to_plot = [od_pairs_elastic["cancel rate air"] * 100]
ax = sns.violinplot(data_to_plot, linewidth=0.7, width=0.8,
                    palette=["mediumblue", "crimson", "darkorange"])
ax.set_xlabel('Air')
ax.set_ylabel('Trip Cancel Rate [%]')
plt.savefig("C:/Users/super/Documents/Masterarbeit/Model/Final_Results_22_12_2022/Figures/violin_plot_trip_cancel_rate_air.png", dpi=1200)

# Violin plot trip cancel rate rail and car
fig = plt.figure(figsize=(10, 4))
data_to_plot = [od_pairs_elastic["cancel rate rail"] * 100, od_pairs_elastic["cancel rate car"] *100]
ax = sns.violinplot(data_to_plot, linewidth=0.7, width=0.9,
                    palette=["crimson", "darkorange"])
ax.set_xlabel('Rail                                                       Car')
ax.set_ylabel('Trip Cancel Rate [%]')
#ax.set_ylim(0, 30)
plt.savefig("C:/Users/super/Documents/Masterarbeit/Model/Final_Results_22_12_2022/Figures/violin_plot_trip_cancel_rate_rail_and_car.png", dpi=1200)


# Stream Graph
distance_list = []
modal_share_air_list = []
modal_share_rail_list = []
modal_share_car_list = []
modal_share_air_cancelled_list = []
modal_share_rail_cancelled_list = []
modal_share_car_cancelled_list = []

for i in range(20):
    d_min = i * 100
    d_max = i * 100 + 100
    data_to_plot = od_pairs_elastic[od_pairs_elastic["distance air"] < d_max]
    data_to_plot = data_to_plot[data_to_plot["distance air"] >= d_min]
    if len(data_to_plot) > 0:
        avg_modal_share_air = statistics.mean(data_to_plot["new modal share air"] * (1 - data_to_plot["cancel rate air"]))# * data_to_plot["new volume"]) / statistics.mean(data_to_plot["new volume"])
        avg_modal_share_rail = statistics.mean(data_to_plot["new modal share rail"] * (1 - data_to_plot["cancel rate air"]))# * data_to_plot["new volume"]) / statistics.mean(data_to_plot["new volume"])
        avg_modal_share_car = statistics.mean(data_to_plot["new modal share car"] * (1 - data_to_plot["cancel rate air"]))# * data_to_plot["new volume"]) / statistics.mean(data_to_plot["new volume"])
        avg_modal_share_air_cancelled = statistics.mean(data_to_plot["new modal share air"] * data_to_plot["cancel rate air"])# * data_to_plot["new volume"]) / statistics.mean(data_to_plot["new volume"])
        avg_modal_share_rail_cancelled = statistics.mean(data_to_plot["new modal share air"] * data_to_plot["cancel rate rail"])# * data_to_plot["new volume"]) / statistics.mean(data_to_plot["new volume"])
        avg_modal_share_car_cancelled = statistics.mean(data_to_plot["new modal share air"] * data_to_plot["cancel rate car"])# * data_to_plot["new volume"]) / statistics.mean(data_to_plot["new volume"])
        modal_share_air_list.append(avg_modal_share_air)
        modal_share_rail_list.append(avg_modal_share_rail)
        modal_share_car_list.append(avg_modal_share_car)
        modal_share_air_cancelled_list.append(avg_modal_share_air_cancelled)
        modal_share_rail_cancelled_list.append(avg_modal_share_rail_cancelled)
        modal_share_car_cancelled_list.append(avg_modal_share_car_cancelled)
        distance_list.append((d_min + d_max) / 2)
modal_shares = [modal_share_air_list, modal_share_rail_list, modal_share_car_list, modal_share_air_cancelled_list, modal_share_rail_cancelled_list, modal_share_car_cancelled_list]
fig, ax = plt.subplots(figsize=(10, 5))
ax.stackplot(distance_list, modal_shares, baseline="sym", colors=["mediumblue", "crimson", "darkorange", "midnightblue", "darkred", "saddlebrown"])
plt.xlabel('Air Distance [km]')
plt.ylabel('Modal Share under TMC')
plt.ylim(-0.5,0.5)
p1 = Rectangle((0, 0), 1, 1, fc="mediumblue")
p2 = Rectangle((0, 0), 1, 1, fc="crimson")
p3 = Rectangle((0, 0), 1, 1, fc="darkorange")
p4 = Rectangle((0, 0), 1, 1, fc="midnightblue")
p5 = Rectangle((0, 0), 1, 1, fc="darkred")
p6 = Rectangle((0, 0), 1, 1, fc="saddlebrown")
plt.legend([p1, p2, p3, p4, p5, p6], ["air", "rail", "car", "air cancelled", "rail cancelled", "car cancelled"],loc="center left", bbox_to_anchor=(1, 0.5))
plt.savefig("C:/Users/super/Documents/Masterarbeit/Model/Final_Results_22_12_2022/Figures/modal_share_stream_graph_elastic.png", dpi=1200)



################################################# 15. Maps ####################################################

########each city one data point
maps_city = pd.DataFrame()
city_list = []

number_of_connections_outbound_list = []
number_of_connections_inbound_list = []

pax_volume_list = []
pax_air_volume_list = []

avg_TC_air_outbound_list = []
avg_TC_air_inbound_list = []
avg_TC_rail_outbound_list = []
avg_TC_rail_inbound_list = []
avg_TC_car_outbound_list = []
avg_TC_car_inbound_list = []

avg_initial_modal_share_air_list = []
avg_initial_modal_share_rail_list = []
avg_initial_modal_share_car_list = []
avg_new_modal_share_air_list = []
avg_new_modal_share_rail_list = []
avg_new_modal_share_car_list = []
avg_modal_shift_air_list = []
avg_modal_shift_rail_list = []
avg_modal_shift_car_list = []

for i in range(len(cities)):
    city_list.append(cities["city"].iloc[i])

    number_of_connections_outbound = 0
    number_of_connections_inbound = 0

    sum_TC_air_outbound = 0
    sum_TC_air_inbound = 0
    sum_TC_rail_outbound = 0
    sum_TC_rail_inbound = 0
    sum_TC_car_outbound = 0
    sum_TC_car_inbound = 0

    sum_initial_modal_share_air = 0
    sum_initial_modal_share_rail = 0
    sum_initial_modal_share_car = 0

    sum_new_modal_share_air = 0
    sum_new_modal_share_rail = 0
    sum_new_modal_share_car = 0

    sum_modal_shift_air = 0
    sum_modal_shift_rail = 0
    sum_modal_shift_car = 0

    sum_pax_volume = 0
    sum_pax_air_volume = 0

    for j in range(len(od_pairs)):
        if cities["city"].iloc[i] == od_pairs["origin"].iloc[j]:
            number_of_connections_outbound = number_of_connections_outbound + 1
            sum_TC_air_outbound = sum_TC_air_outbound + od_pairs["TC air"].iloc[j]
            sum_TC_rail_outbound = sum_TC_rail_outbound + od_pairs["TC rail"].iloc[j]
            sum_TC_car_outbound = sum_TC_car_outbound + od_pairs["TC car"].iloc[j]

            sum_initial_modal_share_air = sum_initial_modal_share_air + od_pairs["modal share air"].iloc[j]
            sum_initial_modal_share_rail = sum_initial_modal_share_rail + od_pairs["modal share rail"].iloc[j]
            sum_initial_modal_share_car = sum_initial_modal_share_car + od_pairs["modal share car"].iloc[j]

            sum_new_modal_share_air = sum_new_modal_share_air + od_pairs["new modal share air"].iloc[j]
            sum_new_modal_share_rail = sum_new_modal_share_rail + od_pairs["new modal share rail"].iloc[j]
            sum_new_modal_share_car = sum_new_modal_share_car + od_pairs["new modal share car"].iloc[j]

            sum_modal_shift_air = sum_modal_shift_air + od_pairs["modal shift air"].iloc[j]
            sum_modal_shift_rail = sum_modal_shift_rail + od_pairs["modal shift rail"].iloc[j]
            sum_modal_shift_car = sum_modal_shift_car + od_pairs["modal shift car"].iloc[j]

            sum_pax_volume = sum_pax_volume + od_pairs["volume"].iloc[j]
            sum_pax_air_volume = sum_pax_air_volume + od_pairs["air volume"].iloc[j]

        if cities["city"].iloc[i] == od_pairs["destination"].iloc[j]:
            number_of_connections_inbound = number_of_connections_inbound + 1
            sum_TC_air_inbound = sum_TC_air_inbound + od_pairs["TC air"].iloc[j]
            sum_TC_rail_inbound = sum_TC_rail_inbound + od_pairs["TC rail"].iloc[j]
            sum_TC_car_inbound = sum_TC_car_inbound + od_pairs["TC car"].iloc[j]

            sum_initial_modal_share_air = sum_initial_modal_share_air + od_pairs["modal share air"].iloc[j]
            sum_initial_modal_share_rail = sum_initial_modal_share_rail + od_pairs["modal share rail"].iloc[j]
            sum_initial_modal_share_car = sum_initial_modal_share_car + od_pairs["modal share car"].iloc[j]

            sum_new_modal_share_air = sum_new_modal_share_air + od_pairs["new modal share air"].iloc[j]
            sum_new_modal_share_rail = sum_new_modal_share_rail + od_pairs["new modal share rail"].iloc[j]
            sum_new_modal_share_car = sum_new_modal_share_car + od_pairs["new modal share car"].iloc[j]

            sum_modal_shift_air = sum_modal_shift_air + od_pairs["modal shift air"].iloc[j]
            sum_modal_shift_rail = sum_modal_shift_rail + od_pairs["modal shift rail"].iloc[j]
            sum_modal_shift_car = sum_modal_shift_car + od_pairs["modal shift car"].iloc[j]

            sum_pax_volume = round(sum_pax_volume + od_pairs["volume"].iloc[j])
            sum_pax_air_volume = round(sum_pax_air_volume + od_pairs["air volume"].iloc[j])

    number_of_connections_outbound_list.append(number_of_connections_outbound)
    number_of_connections_inbound_list.append(number_of_connections_inbound)

    pax_volume_list.append(sum_pax_volume)
    pax_air_volume_list.append(sum_pax_air_volume)

    if number_of_connections_outbound > 0:
        avg_TC_air_outbound = round(sum_TC_air_outbound / number_of_connections_outbound, 2)
        avg_TC_air_outbound_list.append(avg_TC_air_outbound)
        avg_TC_rail_outbound = round(sum_TC_rail_outbound / number_of_connections_outbound, 2)
        avg_TC_rail_outbound_list.append(avg_TC_rail_outbound)
        avg_TC_car_outbound = round(sum_TC_car_outbound / number_of_connections_outbound, 2)
        avg_TC_car_outbound_list.append(avg_TC_car_outbound)
    else:
        avg_TC_air_outbound_list.append("na")
        avg_TC_rail_outbound_list.append("na")
        avg_TC_car_outbound_list.append("na")

    if number_of_connections_inbound > 0:
        avg_TC_air_inbound = round(sum_TC_air_inbound / number_of_connections_inbound, 2)
        avg_TC_air_inbound_list.append(avg_TC_air_inbound)
        avg_TC_rail_inbound = round(sum_TC_rail_inbound / number_of_connections_inbound, 2)
        avg_TC_rail_inbound_list.append(avg_TC_rail_inbound)
        avg_TC_car_inbound = round(sum_TC_car_inbound / number_of_connections_inbound, 2)
        avg_TC_car_inbound_list.append(avg_TC_car_inbound)
    else:
        avg_TC_air_inbound_list.append("na")
        avg_TC_rail_inbound_list.append("na")
        avg_TC_car_inbound_list.append("na")

    if (number_of_connections_outbound + number_of_connections_inbound) > 0:
        avg_initial_modal_share_air = round(
            sum_initial_modal_share_air / (number_of_connections_outbound + number_of_connections_inbound), 4)
        avg_initial_modal_share_air_list.append(avg_initial_modal_share_air)
        avg_initial_modal_share_rail = round(
            sum_initial_modal_share_rail / (number_of_connections_outbound + number_of_connections_inbound), 4)
        avg_initial_modal_share_rail_list.append(avg_initial_modal_share_rail)
        avg_initial_modal_share_car = round(
            sum_initial_modal_share_car / (number_of_connections_outbound + number_of_connections_inbound), 4)
        avg_initial_modal_share_car_list.append(avg_initial_modal_share_car)

        avg_new_modal_share_air = round(
            sum_new_modal_share_air / (number_of_connections_outbound + number_of_connections_inbound), 4)
        avg_new_modal_share_air_list.append(avg_new_modal_share_air)
        avg_new_modal_share_rail = round(
            sum_new_modal_share_rail / (number_of_connections_outbound + number_of_connections_inbound), 4)
        avg_new_modal_share_rail_list.append(avg_new_modal_share_rail)
        avg_new_modal_share_car = round(
            sum_new_modal_share_car / (number_of_connections_outbound + number_of_connections_inbound), 4)
        avg_new_modal_share_car_list.append(avg_new_modal_share_car)

        avg_modal_shift_air = round(
            sum_modal_shift_air / (number_of_connections_outbound + number_of_connections_inbound), 4)
        avg_modal_shift_air_list.append(avg_modal_shift_air)
        avg_modal_shift_rail = round(
            sum_modal_shift_rail / (number_of_connections_outbound + number_of_connections_inbound), 4)
        avg_modal_shift_rail_list.append(avg_modal_shift_rail)
        avg_modal_shift_car = round(
            sum_modal_shift_car / (number_of_connections_outbound + number_of_connections_inbound), 4)
        avg_modal_shift_car_list.append(avg_modal_shift_car)

    else:
        avg_initial_modal_share_air_list.append("na")
        avg_initial_modal_share_rail_list.append("na")
        avg_initial_modal_share_car_list.append("na")

        avg_new_modal_share_air_list.append("na")
        avg_new_modal_share_rail_list.append("na")
        avg_new_modal_share_car_list.append("na")

        avg_modal_shift_air_list.append("na")
        avg_modal_shift_rail_list.append("na")
        avg_modal_shift_car_list.append("na")

maps_city["city"] = city_list

maps_city["number of connections outbound"] = number_of_connections_outbound_list
maps_city["number of connections inbound"] = number_of_connections_inbound_list
maps_city["number of connections"] = maps_city["number of connections outbound"] + maps_city[
    "number of connections inbound"]

maps_city["pax volume"] = pax_volume_list
maps_city["pax air volume"] = pax_air_volume_list

maps_city["avg TC air outbound"] = avg_TC_air_outbound_list
maps_city["avg TC air inbound"] = avg_TC_air_inbound_list
maps_city["avg TC rail outbound"] = avg_TC_rail_outbound_list
maps_city["avg TC rail inbound"] = avg_TC_rail_inbound_list
maps_city["avg TC car outbound"] = avg_TC_car_outbound_list
maps_city["avg TC car inbound"] = avg_TC_car_inbound_list

maps_city["avg initial modal share air"] = avg_initial_modal_share_air_list
maps_city["avg initial modal share rail"] = avg_initial_modal_share_rail_list
maps_city["avg initial modal share car"] = avg_initial_modal_share_car_list

maps_city["avg new modal share air"] = avg_new_modal_share_air_list
maps_city["avg new modal share rail"] = avg_new_modal_share_rail_list
maps_city["avg new modal share car"] = avg_new_modal_share_car_list

maps_city["avg modal shift air"] = avg_modal_shift_air_list
maps_city["avg modal shift rail"] = avg_modal_shift_rail_list
maps_city["avg modal shift car"] = avg_modal_shift_car_list

maps_city["latitude"] = maps_city["city"]
maps_city["longitude"] = maps_city["city"]
for i in range(len(coordinates)):
    for j in range(len(maps_city)):
        if coordinates["city"].iloc[i] == maps_city["city"].iloc[j]:
            maps_city["latitude"].iloc[j] = coordinates["latitude"].iloc[i]
            maps_city["longitude"].iloc[j] = coordinates["longitude"].iloc[i]

maps_city.to_csv("C:/Users/super/Documents/Masterarbeit/Model/Final_Results_22_12_2022/maps_city.csv")
maps_city = pd.read_csv(
    "C:/Users/super/Documents/Masterarbeit/Model/Final_Results_22_12_2022/maps_city.csv", sep=",")

###########Aggregated map

#Pax Flow
# Set up base map
# Set up base map
fig = plt.figure(figsize=(55, 45))
m = Basemap(projection='lcc', resolution='l', area_thresh=1000,
            lon_0=10.6, lat_0=48.9, lat_1=36.9, lat_2=60.9,
            width=3.6E6, height=2.8E6)
# draw map elements
#m.drawcoastlines(color='#999999')
m.fillcontinents(color='#333333', lake_color='#999999')
m.drawmapboundary(fill_color='#999999')
m.drawcountries(color='#999999', linewidth=0.5)

cities_used = np.unique(od_pairs_elastic[['origin', 'destination']].values)
# import long lat as m attribute
for i in range(len(cities)):
    lon = cities["longitude"].iloc[i]
    lat = cities["latitude"].iloc[i]
    xpt, ypt = m(lon, lat)
    #dot_size = cities["pax flow total"].iloc[i] / 4000000
    name = cities["city"].iloc[i]
    dot_size = math.sqrt(cities["pax flow total"].iloc[i]) / 500
    lonpt, latpt = m(xpt, ypt, inverse=True)
    if name in cities_used:
        color = "darkturquoise"
    else:
        color = "lightcyan"
    m.plot(xpt, ypt, 'bo', markersize=dot_size, color=color)
    plt.text(xpt+(100000*dot_size) ** (3/4), ypt+(100000*dot_size) ** (3/4), name, fontsize=4, color = "white")
fig.savefig("C:/Users/super/Documents/Masterarbeit/Model/Final_Results_22_12_2022/Figures/pax_flow_total_per_city3.png",dpi=1200)

#plt.legend(labelspacing=1, loc='center left', bbox_to_anchor=(-0.03, 0.79), fontsize=50, frameon=True, borderpad=1,title="Passenger volume per city", title_fontsize=7, framealpha=0.8)


# plt.text(xpt + 10000, ypt + 10000, cities["city"].iloc[i], color= "white", size=6)

#Number of Connections
# Set up base map
fig = plt.figure(figsize=(55, 45))
m = Basemap(projection='lcc', resolution='l', area_thresh=1000,
            lon_0=10.6, lat_0=48.9, lat_1=36.9, lat_2=60.9,
            width=3.6E6, height=2.8E6)
# draw map elements
#m.drawcoastlines(color='#999999')
m.fillcontinents(color='#333333', lake_color='#999999')
m.drawmapboundary(fill_color='#999999')
m.drawcountries(color='#999999', linewidth=0.5)
# import long lat as m attribute
for i in range(len(cities)):
    lon = cities["longitude"].iloc[i]
    lat = cities["latitude"].iloc[i]
    xpt, ypt = m(lon, lat)
    name = cities["city"].iloc[i]
    if maps_city["number of connections"].iloc[i] > 0:
        dot_size = math.sqrt(maps_city["number of connections"].iloc[i]) / 1.5
        color = "deepskyblue"
    else:
        dot_size = 2
        color = "white"
    lonpt, latpt = m(xpt, ypt, inverse=True)
    m.plot(xpt, ypt, 'bo', markersize=dot_size, color=color)
    plt.text(xpt+(100000*dot_size) ** (3/4), ypt+(100000*dot_size) ** (3/4), name, fontsize=4, color = "white")
fig.savefig("C:/Users/super/Documents/Masterarbeit/Model/Final_Results_22_12_2022/Figures/number_of_connections3.png",dpi=1200)




#Relative change in pax volume:
rel_change_pax_volume_rail = []
for i in range(len(od_pairs_elastic)):
    old_value = od_pairs_elastic["volume"].iloc[i] * od_pairs_elastic["modal share rail"].iloc[i]
    new_value = od_pairs_elastic["new volume"].iloc[i] * od_pairs_elastic["new modal share rail"].iloc[i]
    rel_change_pax_volume_rail.append((new_value - old_value) / old_value)
od_pairs_elastic["rel change pax volume rail"] = rel_change_pax_volume_rail


##################Connections to one city

# Map with initial modal share air (width of the edges), and modal shift air = new modal share - initial modal share (color of the edges). Red = high modal shift, white = low modal shift
hub = "Zurich"
attribute = "modal share air"

# Set up base map
fig = plt.figure(figsize=(55, 45))
m = Basemap(projection='lcc', resolution='l', area_thresh=1000,
            lon_0=9.6, lat_0=48.4, lat_1=36.4, lat_2=60.4,
            width=3.3E6, height=2.6E6)
# draw map elements
#m.drawcoastlines(color='#999999')
m.fillcontinents(color='#333333', lake_color='#999999')
m.drawmapboundary(fill_color='#999999')
m.drawcountries(color='#999999', linewidth=0.5)

# Draw hub city
lon_hub = od_pairs_elastic.loc[od_pairs_elastic["destination"] == hub]["long destination"].values[0]
lat_hub = od_pairs_elastic.loc[od_pairs_elastic["destination"] == hub]["lat destination"].values[0]
xpt_hub, ypt_hub = m(lon_hub, lat_hub)
lonpt_hub, latpt_hub = m(xpt_hub, ypt_hub, inverse=True)
m.plot(xpt_hub, ypt_hub, 'bo', markersize=3, color="palegreen")
#plt.legend(loc='center left', bbox_to_anchor=(-0.03, 0.79), fontsize=50, frameon=True, borderpad=1, title="Passenger volume per city", title_fontsize=7, framealpha=0.8)

max_value = abs(od_pairs_elastic[attribute]).max()
min_value = abs(od_pairs_elastic[attribute]).min()
# Draw other cities
for i in range(len(od_pairs_elastic)):
    if od_pairs_elastic["origin"].iloc[i] == hub:
        lon = od_pairs_elastic["long destination"].iloc[i]
        lat = od_pairs_elastic["lat destination"].iloc[i]
        xpt, ypt = m(lon, lat)
        lonpt, latpt = m(xpt, ypt, inverse=True)
        G = nx.Graph()
        G.add_edge('a', 'b')
        pos = {}
        pos['a'] = (xpt_hub, ypt_hub)
        pos['b'] = (xpt, ypt)
        norm_value = (abs(od_pairs_elastic[attribute].iloc[i]) - min_value) / (max_value - min_value)            #(abs(od_pairs_elastic["modal shift air"].iloc[i]) - min_value) / max_value
        color = colorsys.hsv_to_rgb(0, norm_value, 1)
        # nx.draw_networkx(G, pos, width = od_pairs["modal share air"].iloc[i] / statistics.mean(od_pairs["modal share air"]) * 2, node_size=0, node_color='blue',edge_color="white", with_labels=False, alpha=0.5)
        #nx.draw_networkx(G, pos, width=od_pairs_elastic["new volume"].iloc[i] / 100000, node_size=0, node_color='blue', edge_color=color, with_labels=False, alpha=0.7)
        nx.draw_networkx(G, pos, width = math.sqrt(od_pairs_elastic["new modal share rail"].iloc[i] * 50), node_size=0, node_color='blue',
                         edge_color=color, with_labels=False, alpha=0.7)
        m.plot(xpt, ypt, 'bo', markersize=3, color="yellowgreen")

fig.savefig("C:/Users/super/Documents/Masterarbeit/Model/Final_Results_22_12_2022/Figures/Amsterdam_rel_change_modal_share_air.png",dpi=1200)

# plt.legend(labelspacing=1, loc='center left', bbox_to_anchor=(0, 0.7), fontsize=50, frameon=True,borderpad=1, title="Modal Shift", title_fontsize=7, framealpha=0.8)


###Corridor
# Map with initial modal share air (width of the edges), and modal shift air = new modal share - initial modal share (color of the edges). Red = high modal shift, white = low modal shift
hub = "Zurich"
attribute = "rel change modal share air"

# Set up base map
fig = plt.figure(figsize=(15, 7))
m = Basemap(projection='lcc', resolution='l', area_thresh=1000,
            lon_0=10, lat_0=49, lat_1=42, lat_2=61,
            width=1.5E6, height=1E6)
# draw map elements
#m.drawcoastlines(color='#999999')
m.fillcontinents(color='#333333', lake_color='#999999')
m.drawmapboundary(fill_color='#999999')
m.drawcountries(color='#999999', linewidth=0.5)

# Draw hub city
lon_hub = od_pairs_elastic.loc[od_pairs_elastic["destination"] == hub]["long destination"].values[0]
lat_hub = od_pairs_elastic.loc[od_pairs_elastic["destination"] == hub]["lat destination"].values[0]
xpt_hub, ypt_hub = m(lon_hub, lat_hub)
lonpt_hub, latpt_hub = m(xpt_hub, ypt_hub, inverse=True)
m.plot(xpt_hub, ypt_hub, 'bo', markersize=3, color="palegreen")
plt.text(xpt_hub + 10000, ypt_hub + 10000, hub, fontsize=5, color="palegreen")
#plt.legend(loc='center left', bbox_to_anchor=(-0.03, 0.79), fontsize=50, frameon=True, borderpad=1, title="Passenger volume per city", title_fontsize=7, framealpha=0.8)

max_value = abs(od_pairs_elastic[attribute]).max()
min_value = abs(od_pairs_elastic[attribute]).min()
# Draw other cities
for i in range(len(od_pairs_elastic)):
    if od_pairs_elastic["origin"].iloc[i] == hub:
        if od_pairs_elastic["long destination"].iloc[i] > -1 and od_pairs_elastic["long destination"].iloc[i] < 15:
            if od_pairs_elastic["lat destination"].iloc[i] > 45 and od_pairs_elastic["lat destination"].iloc[i] < 50:
                lon = od_pairs_elastic["long destination"].iloc[i]
                lat = od_pairs_elastic["lat destination"].iloc[i]
                xpt, ypt = m(lon, lat)
                lonpt, latpt = m(xpt, ypt, inverse=True)
                G = nx.Graph()
                G.add_edge('a', 'b')
                pos = {}
                pos['a'] = (xpt_hub, ypt_hub)
                pos['b'] = (xpt, ypt)
                norm_value = abs((od_pairs_elastic[attribute].iloc[i] - min_value) / (max_value - min_value))
                color = colorsys.hsv_to_rgb(0, norm_value, 1)
                nx.draw_networkx(G, pos, width=3, node_size=0, node_color='blue',
                                 edge_color=color, with_labels=False, alpha=0.7)
                m.plot(xpt, ypt, 'bo', markersize=3, color="yellowgreen")
                plt.text(xpt, ypt, od_pairs_elastic["destination"].iloc[i], fontsize=4, color="yellowgreen")



#Calculate total cancel rate
total_cancel_rate = []
for i in range(len(od_pairs_elastic)):
    total_cancel_rate.append((od_pairs_elastic["new volume"].iloc[i] - od_pairs_elastic["volume"].iloc[i]) / od_pairs_elastic["volume"].iloc[i])
od_pairs_elastic["total cancel rate"] = total_cancel_rate

#######Network Map all OD-Pairs
attribute = "total cancel rate"
# Set up base map
fig = plt.figure(figsize=(55, 45))
m = Basemap(projection='lcc', resolution='l', area_thresh=1000,
            lon_0=9.6 lat_0=48.4, lat_1=36.4, lat_2=60.4,
            width=3.3E6, height=2.6E6)
# draw map elements
#m.drawcoastlines(color='#999999')
m.fillcontinents(color='#333333', lake_color='#999999')
m.drawmapboundary(fill_color='#999999')
m.drawcountries(color='#999999', linewidth=0.5)

max_value = abs(od_pairs_elastic[attribute]).max()
min_value = abs(od_pairs_elastic[attribute]).min()

# Draw routes
for i in range(len(od_pairs_elastic)):
    #if od_pairs_elastic["new modal share air"].iloc[i] > 0.99:
    lon_a = od_pairs_elastic["long origin"].iloc[i]
    lat_a = od_pairs_elastic["lat origin"].iloc[i]
    lon_b = od_pairs_elastic["long destination"].iloc[i]
    lat_b = od_pairs_elastic["lat destination"].iloc[i]
    xpt_a, ypt_a = m(lon_a, lat_a)
    xpt_b, ypt_b = m(lon_b, lat_b)
    m.plot(xpt_a, ypt_a, 'bo', markersize=2, color="yellowgreen")
    m.plot(xpt_b, ypt_b, 'bo', markersize=2, color="yellowgreen")
    #Two directions: Move lines a bit that travel direction is always on the right
    linewidth = 0.2
    if lon_a < lon_b and lat_a < lat_b:
        lon_a = lon_a + linewidth / 10
        lat_a = lat_a - linewidth / 10
        lon_b = lon_b + linewidth / 10
        lat_b = lat_b - linewidth / 10
    elif lon_a < lon_b and lat_a > lat_b:
        lon_a = lon_a - linewidth / 10
        lat_a = lat_a - linewidth / 10
        lon_b = lon_b - linewidth / 10
        lat_b = lat_b - linewidth / 10
    elif lon_a > lon_b and lat_a < lat_b:
        lon_a = lon_a + linewidth / 10
        lat_a = lat_a + linewidth / 10
        lon_b = lon_b + linewidth / 10
        lat_b = lat_b + linewidth / 10
    else:
        lon_a = lon_a - linewidth / 10
        lat_a = lat_a + linewidth / 10
        lon_b = lon_b - linewidth / 10
        lat_b = lat_b + linewidth / 10

    xpt_a, ypt_a = m(lon_a, lat_a)
    xpt_b, ypt_b = m(lon_b, lat_b)
    G = nx.Graph()
    G.add_edge('a', 'b')
    pos = {}
    pos['a'] = (xpt_a, ypt_a)
    pos['b'] = (xpt_b, ypt_b)

    norm_value = (abs(od_pairs_elastic[attribute].iloc[i]) - min_value) / (max_value - min_value)
    color = colorsys.hsv_to_rgb(0, norm_value, 1)
    nx.draw_networkx(G, pos, width = 0.2, node_size=0, node_color='blue',
                     edge_color=color, with_labels=False, alpha=0.7)

fig.savefig("C:/Users/super/Documents/Masterarbeit/Model/Final_Results_22_12_2022/Figures/Map all OD-pairs/total_cancel_rate.png",dpi=1200)



rel_change_TC_air_list = []
for i in range(len(od_pairs_elastic)):
    rel_change_TC_air = (od_pairs_elastic["TC air new"].iloc[i] - od_pairs_elastic["TC air"].iloc[i]) / od_pairs_elastic["TC air"].iloc[i]
    rel_change_TC_air_list.append(rel_change_TC_air)
od_pairs_elastic["rel change TC air"] = rel_change_TC_air_list



od_pairs_category.to_csv("C:/Users/super/Documents/Masterarbeit/Model/Final_Results_22_12_2022/air below 1%.csv")

lower_limit = 0 * 200
upper_limit = lower_limit + 400

share_origin_list = []
share_destination_list = []

#######Network Map categorised OD-Pairs
od_pairs_category = od_pairs_elastic[od_pairs_elastic["TC air"] > 0]
od_pairs_category = od_pairs_category[od_pairs_category["modal shift car"] < 0.8]
od_pairs_category = od_pairs_category[od_pairs_category["modal share car"] < 0.8]
od_pairs_category = od_pairs_category[od_pairs_category["distance air"] < 1200]




print(od_pairs_elastic["volume"].sum())
print(od_pairs_elastic["new volume"].sum())

(od_pairs_elastic["new volume"].sum() - od_pairs_elastic["volume"].sum()) / od_pairs_elastic["volume"].sum()
print(statistics.mean(od_pairs_elastic["total cancel rate"]))

attribute = "modal shift rail"
# Set up base map
fig = plt.figure(figsize=(55, 45))
m = Basemap(projection='lcc', resolution='l', area_thresh=1000,
            lon_0=9.6, lat_0=48.4, lat_1=36.4, lat_2=60.4,
            width=3.3E6, height=2.6E6)
# draw map elements
#m.drawcoastlines(color='#999999')
m.fillcontinents(color='#333333', lake_color='#999999')
m.drawmapboundary(fill_color='#999999')
m.drawcountries(color='#999999', linewidth=0.5)

max_value = (abs(od_pairs_elastic[attribute])).max()
min_value = (abs(od_pairs_elastic[attribute])).min()

print(od_pairs_elastic["cancel rate rail"].min())

for i in range(len(od_pairs_elastic)):
    lon = od_pairs_elastic["long origin"].iloc[i]
    lat = od_pairs_elastic["lat origin"].iloc[i]
    xpt, ypt = m(lon, lat)
    m.plot(xpt, ypt, 'bo', markersize=1, color="mistyrose")

# Draw routes
for i in range(len(od_pairs_category)):
    lon_a = od_pairs_category["long origin"].iloc[i]
    lat_a = od_pairs_category["lat origin"].iloc[i]
    lon_b = od_pairs_category["long destination"].iloc[i]
    lat_b = od_pairs_category["lat destination"].iloc[i]
    xpt_a, ypt_a = m(lon_a, lat_a)
    xpt_b, ypt_b = m(lon_b, lat_b)

    # Number of connections from origin city divided by total connections from origin city
    lat_origin = od_pairs_category["lat origin"].iloc[i]
    share_origin = od_pairs_category["lat origin"].value_counts()[lat_origin] / od_pairs_elastic["lat origin"].value_counts()[lat_origin]
    share_origin_list.append(share_origin)
    # Number of connections to destination city divided by total connections to destination city
    lat_destination = od_pairs_category["lat destination"].iloc[i]
    share_destination =  od_pairs_category["lat destination"].value_counts()[lat_destination] / od_pairs_elastic["lat destination"].value_counts()[lat_destination]
    share_destination_list.append(share_destination)

    # if dot for city smaller than 1, make it to 1
    factor = 15
    marker_size_1 = math.sqrt(share_origin) * factor
    marker_size_2 = math.sqrt(share_destination) * factor
    if marker_size_1 < 1:
        marker_size_1 = 1
    if marker_size_2 < 1:
        marker_size_2 = 1
    m.plot(xpt_a, ypt_a, 'bo', markersize = marker_size_1, color="violet")
    m.plot(xpt_b, ypt_b, 'bo', markersize = marker_size_2, color="violet")

    #Two directions: Move lines a bit that travel direction is always on the right
    linewidth = 0.5
    if lon_a < lon_b and lat_a < lat_b:
        lon_a = lon_a + linewidth / 10
        lat_a = lat_a - linewidth / 10
        lon_b = lon_b + linewidth / 10
        lat_b = lat_b - linewidth / 10
    elif lon_a < lon_b and lat_a > lat_b:
        lon_a = lon_a - linewidth / 10
        lat_a = lat_a - linewidth / 10
        lon_b = lon_b - linewidth / 10
        lat_b = lat_b - linewidth / 10
    elif lon_a > lon_b and lat_a < lat_b:
        lon_a = lon_a + linewidth / 10
        lat_a = lat_a + linewidth / 10
        lon_b = lon_b + linewidth / 10
        lat_b = lat_b + linewidth / 10
    else:
        lon_a = lon_a - linewidth / 10
        lat_a = lat_a + linewidth / 10
        lon_b = lon_b - linewidth / 10
        lat_b = lat_b + linewidth / 10

    xpt_a, ypt_a = m(lon_a, lat_a)
    xpt_b, ypt_b = m(lon_b, lat_b)
    G = nx.Graph()
    G.add_edge('a', 'b')
    pos = {}
    pos['a'] = (xpt_a, ypt_a)
    pos['b'] = (xpt_b, ypt_b)

    norm_value = (abs(od_pairs_category[attribute].iloc[i]) - min_value) / (max_value - min_value)
    color = colorsys.hsv_to_rgb(1, norm_value, 1)     #Air: First value 1.6, Rail: First value 1, Car: First value 1.1
    nx.draw_networkx(G, pos, width = linewidth * 0.3, node_size=0, node_color='blue',
                     edge_color=color, with_labels=False, alpha=0.7)

od_pairs_category["share_origin"] = share_origin_list
od_pairs_category["share destination"] = share_destination_list

fig.savefig("C:/Users/super/Documents/Masterarbeit/Model/Final_Results_22_12_2022/decrease_modal_share_car.png",dpi=1200)

print(od_pairs_category["rel change modal share rail"].max())




#######Focus routes

focus_routes = pd.read_csv("C:/Users/super/Documents/Masterarbeit/Model/focus_routes.csv", sep=";")

attribute = "modal shift rail"
# Set up base map
fig = plt.figure(figsize=(55, 45))
m = Basemap(projection='lcc', resolution='l', area_thresh=1000,
            lon_0=6, lat_0=46.7, lat_1=34.7, lat_2=58.7,
            width=1.9E6, height=1.5E6)
# draw map elements
#m.drawcoastlines(color='#999999')
m.fillcontinents(color='#333333', lake_color='#999999')
m.drawmapboundary(fill_color='#999999')
m.drawcountries(color='#999999', linewidth=0.5)

max_value = od_pairs_elastic[attribute].max()
min_value = od_pairs_elastic[attribute].min()

for i in range(len(cities)):
    lon = cities["longitude"].iloc[i]
    lat = cities["latitude"].iloc[i]
    xpt, ypt = m(lon, lat)
    name = cities["city"].iloc[i]
    if focus_routes["origin"].str.contains(name).any() or focus_routes["destination"].str.contains(name).any():
        m.plot(xpt, ypt, 'bo', markersize=5, color="mistyrose")
        if cities["city"].iloc[i] == "Amsterdam":
            plt.text(xpt - 200000, ypt, name, fontsize=10, color="mistyrose")
        elif cities["city"].iloc[i] == "Paris":
            plt.text(xpt - 100000, ypt, name, fontsize=10, color="mistyrose")
        elif cities["city"].iloc[i] == "Madrid":
            plt.text(xpt - 130000, ypt, name, fontsize=10, color="mistyrose")
        elif cities["city"].iloc[i] == "Zurich":
            plt.text(xpt + 10000, ypt - 40000, name, fontsize=10, color="mistyrose")
        else:
            plt.text(xpt + 20000, ypt, name, fontsize=10, color="mistyrose")
    else:
        m.plot(xpt, ypt, 'bo', markersize=1, color="mistyrose")

# Draw routes
for i in range(len(focus_routes)):
    lon_a = focus_routes["long origin"].iloc[i]
    lat_a = focus_routes["lat origin"].iloc[i]
    lon_b = focus_routes["long destination"].iloc[i]
    lat_b = focus_routes["lat destination"].iloc[i]
    xpt_a, ypt_a = m(lon_a, lat_a)
    xpt_b, ypt_b = m(lon_b, lat_b)

    # Number of connections from origin city divided by total connections from origin city
    lat_origin = focus_routes["lat origin"].iloc[i]
    share_origin = focus_routes["lat origin"].value_counts()[lat_origin] / focus_routes["lat origin"].value_counts()[lat_origin]
    share_origin_list.append(share_origin)
    # Number of connections to destination city divided by total connections to destination city
    lat_destination = focus_routes["lat destination"].iloc[i]
    share_destination =  focus_routes["lat destination"].value_counts()[lat_destination] / focus_routes["lat destination"].value_counts()[lat_destination]
    share_destination_list.append(share_destination)

    # if dot for city smaller than 1, make it to 1
    factor = 5
    marker_size_1 = math.sqrt(share_origin) * factor
    marker_size_2 = math.sqrt(share_destination) * factor
    if marker_size_1 < 1:
        marker_size_1 = 1
    if marker_size_2 < 1:
        marker_size_2 = 1
    m.plot(xpt_a, ypt_a, 'bo', markersize = marker_size_1, color="white")
    m.plot(xpt_b, ypt_b, 'bo', markersize = marker_size_2, color="white")

    #Two directions: Move lines a bit that travel direction is always on the right
    linewidth = 0.5
    if lon_a < lon_b and lat_a < lat_b:
        lon_a = lon_a + linewidth / 10
        lat_a = lat_a - linewidth / 10
        lon_b = lon_b + linewidth / 10
        lat_b = lat_b - linewidth / 10
    elif lon_a < lon_b and lat_a > lat_b:
        lon_a = lon_a - linewidth / 10
        lat_a = lat_a - linewidth / 10
        lon_b = lon_b - linewidth / 10
        lat_b = lat_b - linewidth / 10
    elif lon_a > lon_b and lat_a < lat_b:
        lon_a = lon_a + linewidth / 10
        lat_a = lat_a + linewidth / 10
        lon_b = lon_b + linewidth / 10
        lat_b = lat_b + linewidth / 10
    else:
        lon_a = lon_a - linewidth / 10
        lat_a = lat_a + linewidth / 10
        lon_b = lon_b - linewidth / 10
        lat_b = lat_b + linewidth / 10

    xpt_a, ypt_a = m(lon_a, lat_a)
    xpt_b, ypt_b = m(lon_b, lat_b)
    G = nx.Graph()
    G.add_edge('a', 'b')
    pos = {}
    pos['a'] = (xpt_a, ypt_a)
    pos['b'] = (xpt_b, ypt_b)

    norm_value = (abs(focus_routes[attribute].iloc[i]) - min_value) / (max_value - min_value)
    color = colorsys.hsv_to_rgb(1, norm_value, 1)     #Air: First value 1.6, Rail: First value 1, Car: First value 1.1
    nx.draw_networkx(G, pos, width = linewidth * 5, node_size=0, node_color='blue',
                     edge_color=color, with_labels=False, alpha=0.7)

od_pairs_category["share_origin"] = share_origin_list
od_pairs_category["share destination"] = share_destination_list

fig.savefig("C:/Users/super/Documents/Masterarbeit/Model/Final_Results_22_12_2022/focus_routes_basemap.png",dpi=1200)


###########Focus cities
city = "Brussels"
#Average modal share
k = 0
air_share_sum = 0
rail_share_sum = 0
car_share_sum = 0
new_air_share_sum = 0
new_rail_share_sum = 0
new_car_share_sum = 0
for i in range(len(od_pairs_elastic)):
    if od_pairs_elastic["origin"].iloc[i] == city or od_pairs_elastic["destination"].iloc[i] == city:
        k = k + 1
        air_share_sum = air_share_sum + od_pairs_elastic["modal share air"].iloc[i]
        rail_share_sum = rail_share_sum + od_pairs_elastic["modal share rail"].iloc[i]
        car_share_sum = car_share_sum + od_pairs_elastic["modal share car"].iloc[i]
        new_air_share_sum = new_air_share_sum + od_pairs_elastic["new modal share air"].iloc[i]
        new_rail_share_sum = new_rail_share_sum + od_pairs_elastic["new modal share rail"].iloc[i]
        new_car_share_sum = new_car_share_sum + od_pairs_elastic["new modal share car"].iloc[i]

air_share_sum = air_share_sum / k
rail_share_sum = rail_share_sum / k
car_share_sum = car_share_sum / k
new_air_share_sum = new_air_share_sum / k
new_rail_share_sum = new_rail_share_sum / k
new_car_share_sum = new_car_share_sum / k
print(air_share_sum)
print(rail_share_sum)
print(car_share_sum)
print(new_air_share_sum)
print(new_rail_share_sum)
print(new_car_share_sum)